@@ -16,7 +16,7 @@
 --51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 local function create_world_formspec(dialogdata)
-	local mapgens = {"v5", "v6", "v7", "singlenode"}
+	local mapgens = core.get_mapgen_names()
 
 	local current_seed = core.setting_get("fixed_map_seed") or ""
 	local current_mg   = core.setting_get("mg_name")
@@ -90,6 +90,8 @@ local function create_world_buttonhandler(this, fields)
 
 			local message = nil
 
+			core.setting_set("fixed_map_seed", fields["te_seed"])
+
 			if not menudata.worldlist:uid_exists_raw(worldname) then
 				core.setting_set("mg_name",fields["dd_mapgen"])
 				message = core.create_world(worldname,gameindex)
@@ -97,8 +99,6 @@ local function create_world_buttonhandler(this, fields)
 				message = fgettext("A world named \"$1\" already exists", worldname)
 			end
 
-			core.setting_set("fixed_map_seed", fields["te_seed"])
-
 			if message ~= nil then
 				gamedata.errormessage = message
 			else
