@@ -108,9 +108,28 @@ function doors.register_door(name, def)
 		end
 	end
 
+	local function check_and_blast(pos, name)
+		local node = minetest.get_node(pos)
+		if node.name == name then
+			minetest.remove_node(pos)
+		end
+	end
+
+	local function make_on_blast(base_name, dir, door_type, other_door_type)
+		if def.only_placer_can_open then
+			return function() end
+		else
+			return function(pos, intensity)
+				check_and_blast(pos, base_name .. door_type)
+				pos.y = pos.y + dir
+				check_and_blast(pos, base_name .. other_door_type)
+			end
+		end
+	end
+
 	local function on_rightclick(pos, dir, check_name, replace, replace_dir, params)
 		pos.y = pos.y+dir
-		if not minetest.get_node(pos).name == check_name then
+		if minetest.get_node(pos).name ~= check_name then
 			return
 		end
 		local p2 = minetest.get_node(pos).param2
@@ -144,10 +163,38 @@ function doors.register_door(name, def)
 		return meta:get_string("doors_owner") == pn
 	end
 
+	local function on_rotate(pos, node, dir, user, check_name, mode, new_param2)
+		if not check_player_priv(pos, user) then
+			return false
+		end
+		if mode ~= screwdriver.ROTATE_FACE then
+			return false
+		end
+
+		pos.y = pos.y + dir
+		if not minetest.get_node(pos).name == check_name then
+			return false
+		end
+		if minetest.is_protected(pos, user:get_player_name()) then
+			minetest.record_protection_violation(pos, user:get_player_name())
+			return false
+		end
+
+		local node2 = minetest.get_node(pos)
+		node2.param2 = (node2.param2 + 1) % 4
+		minetest.swap_node(pos, node2)
+
+		pos.y = pos.y - dir
+		node.param2 = (node.param2 + 1) % 4
+		minetest.swap_node(pos, node)
+		return true
+	end
+
 	minetest.register_node(name.."_b_1", {
 		tiles = {tb[2], tb[2], tb[2], tb[2], tb[1], tb[1].."^[transformfx"},
 		paramtype = "light",
 		paramtype2 = "facedir",
+		is_ground_content = false,
 		drop = name,
 		drawtype = "nodebox",
 		node_box = {
@@ -171,15 +218,21 @@ function doors.register_door(name, def)
 			end
 		end,
 		
+		on_rotate = function(pos, node, user, mode, new_param2)
+			return on_rotate(pos, node, 1, user, name.."_t_1", mode)
+		end,
+
 		can_dig = check_player_priv,
 		sounds = def.sounds,
-        	sunlight_propagates = def.sunlight
+		sunlight_propagates = def.sunlight,
+		on_blast = make_on_blast(name, 1, "_b_1", "_t_1")
 	})
 
 	minetest.register_node(name.."_t_1", {
 		tiles = {tt[2], tt[2], tt[2], tt[2], tt[1], tt[1].."^[transformfx"},
 		paramtype = "light",
 		paramtype2 = "facedir",
+		is_ground_content = false,
 		drop = "",
 		drawtype = "nodebox",
 		node_box = {
@@ -203,15 +256,21 @@ function doors.register_door(name, def)
 			end
 		end,
 		
+		on_rotate = function(pos, node, user, mode, new_param2)
+			return on_rotate(pos, node, -1, user, name.."_b_1", mode)
+		end,
+
 		can_dig = check_player_priv,
 		sounds = def.sounds,
-        	sunlight_propagates = def.sunlight,
+		sunlight_propagates = def.sunlight,
+		on_blast = make_on_blast(name, -1, "_t_1", "_b_1")
 	})
 
 	minetest.register_node(name.."_b_2", {
 		tiles = {tb[2], tb[2], tb[2], tb[2], tb[1].."^[transformfx", tb[1]},
 		paramtype = "light",
 		paramtype2 = "facedir",
+		is_ground_content = false,
 		drop = name,
 		drawtype = "nodebox",
 		node_box = {
@@ -235,15 +294,21 @@ function doors.register_door(name, def)
 			end
 		end,
 		
+		on_rotate = function(pos, node, user, mode, new_param2)
+			return on_rotate(pos, node, 1, user, name.."_t_2", mode)
+		end,
+
 		can_dig = check_player_priv,
 		sounds = def.sounds,
-        	sunlight_propagates = def.sunlight
+		sunlight_propagates = def.sunlight,
+		on_blast = make_on_blast(name, 1, "_b_2", "_t_2")
 	})
 
 	minetest.register_node(name.."_t_2", {
 		tiles = {tt[2], tt[2], tt[2], tt[2], tt[1].."^[transformfx", tt[1]},
 		paramtype = "light",
 		paramtype2 = "facedir",
+		is_ground_content = false,
 		drop = "",
 		drawtype = "nodebox",
 		node_box = {
@@ -267,9 +332,14 @@ function doors.register_door(name, def)
 			end
 		end,
 		
+		on_rotate = function(pos, node, user, mode, new_param2)
+			return on_rotate(pos, node, -1, user, name.."_b_2", mode)
+		end,
+
 		can_dig = check_player_priv,
 		sounds = def.sounds,
-        	sunlight_propagates = def.sunlight
+		sunlight_propagates = def.sunlight,
+		on_blast = make_on_blast(name, -1, "_t_2", "_b_2")
 	})
 
 end
@@ -369,10 +439,13 @@ function doors.register_trapdoor(name, def)
 		minetest.set_node(pos, {name = newname, param1 = node.param1, param2 = node.param2})
 	end
 
+	def.on_rotate = minetest.get_modpath("screwdriver") and screwdriver.rotate_simple
+
 	-- Common trapdoor configuration
 	def.drawtype = "nodebox"
 	def.paramtype = "light"
 	def.paramtype2 = "facedir"
+	def.is_ground_content = false
 
 	local def_opened = table.copy(def)
 	local def_closed = table.copy(def)
