@@ -2,64 +2,40 @@ farming.register_hoe(":farming:hoe_wood", {
 	description = "Wooden Hoe",
 	inventory_image = "farming_tool_woodhoe.png",
 	max_uses = 30,
-	recipe = {
-		{"group:wood", "group:wood"},
-		{"", "group:stick"},
-		{"", "group:stick"},
-	}
+	material = "group:wood"
 })
 
 farming.register_hoe(":farming:hoe_stone", {
 	description = "Stone Hoe",
 	inventory_image = "farming_tool_stonehoe.png",
 	max_uses = 90,
-	recipe = {
-		{"group:stone", "group:stone"},
-		{"", "group:stick"},
-		{"", "group:stick"},
-	}
+	material = "group:stone"
 })
 
 farming.register_hoe(":farming:hoe_steel", {
 	description = "Steel Hoe",
 	inventory_image = "farming_tool_steelhoe.png",
 	max_uses = 200,
-	recipe = {
-		{"default:steel_ingot", "default:steel_ingot"},
-		{"", "group:stick"},
-		{"", "group:stick"},
-	}
+	material = "default:steel_ingot"
 })
 
 farming.register_hoe(":farming:hoe_bronze", {
 	description = "Bronze Hoe",
 	inventory_image = "farming_tool_bronzehoe.png",
 	max_uses = 220,
-	recipe = {
-		{"default:bronze_ingot", "default:bronze_ingot"},
-		{"", "group:stick"},
-		{"", "group:stick"},
-	}
+	material = "default:bronze_ingot"
 })
 
 farming.register_hoe(":farming:hoe_mese", {
 	description = "Mese Hoe",
 	inventory_image = "farming_tool_mesehoe.png",
 	max_uses = 350,
-	recipe = {
-		{"default:mese_crystal", "default:mese_crystal"},
-		{"", "group:stick"},
-		{"", "group:stick"},
-	}
+	material = "default:mese_crystal"
 })
 
 farming.register_hoe(":farming:hoe_diamond", {
 	description = "Diamond Hoe",
 	inventory_image = "farming_tool_diamondhoe.png",
 	max_uses = 500,
-	recipe = {
-		{"default:diamond", "default:diamond"},
-		{"", "group:stick"},
-		{"", "group:stick"},
-	}
+	material = "default:diamond"
 })
