@@ -20,8 +20,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #ifndef BAN_HEADER
 #define BAN_HEADER
 
-#include <map>
-#include <string>
+#include "util/string.h"
 #include "jthread/jthread.h"
 #include "jthread/jmutex.h"
 #include "exceptions.h"
@@ -43,7 +42,7 @@ public:
 private:
 	JMutex m_mutex;
 	std::string m_banfilepath;
-	std::map<std::string, std::string> m_ips;
+	StringMap m_ips;
 	bool m_modified;
 
 };
