@@ -25,21 +25,17 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "database.h"
 #include "irrlichttypes.h"
 
-class ServerMap;
-
 class Database_Dummy : public Database
 {
 public:
-	Database_Dummy(ServerMap *map);
-	virtual void beginSave();
-	virtual void endSave();
-	virtual bool saveBlock(v3s16 blockpos, std::string &data);
-	virtual std::string loadBlock(v3s16 blockpos);
-	virtual void listAllLoadableBlocks(std::list<v3s16> &dst);
-	virtual int Initialized(void);
-	~Database_Dummy();
+	virtual bool saveBlock(const v3s16 &pos, const std::string &data);
+	virtual std::string loadBlock(const v3s16 &pos);
+	virtual bool deleteBlock(const v3s16 &pos);
+	virtual void listAllLoadableBlocks(std::vector<v3s16> &dst);
+
 private:
-	ServerMap *srvmap;
-	std::map<u64, std::string> m_database;
+	std::map<s64, std::string> m_database;
 };
+
 #endif
+
