@@ -18,12 +18,12 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 */
 
 #include "drawscene.h"
-#include "main.h" // for g_settings
 #include "settings.h"
 #include "clouds.h"
 #include "clientmap.h"
 #include "util/timetaker.h"
 #include "fontengine.h"
+#include "guiscalingfilter.h"
 
 typedef enum {
 	LEFT = -1,
@@ -167,7 +167,7 @@ video::ITexture* draw_image(const v2u32& screensize,
 		image = images[1];
 	else
 		image = images[0];
-	
+
 	driver->setRenderTarget(image, true, true,
 			irr::video::SColor(255,
 					skycolor.getRed(), skycolor.getGreen(), skycolor.getBlue()));
@@ -324,19 +324,19 @@ void draw_sidebyside_3d_mode(Camera& camera, bool show_hud,
 	//makeColorKeyTexture mirrors texture so we do it twice to get it right again
 	driver->makeColorKeyTexture(hudtexture, irr::video::SColor(255, 0, 0, 0));
 
-	driver->draw2DImage(left_image,
+	draw2DImageFilterScaled(driver, left_image,
 			irr::core::rect<s32>(0, 0, screensize.X/2, screensize.Y),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, false);
 
-	driver->draw2DImage(hudtexture,
+	draw2DImageFilterScaled(driver, hudtexture,
 			irr::core::rect<s32>(0, 0, screensize.X/2, screensize.Y),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, true);
 
-	driver->draw2DImage(right_image,
+	draw2DImageFilterScaled(driver, right_image,
 			irr::core::rect<s32>(screensize.X/2, 0, screensize.X, screensize.Y),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, false);
 
-	driver->draw2DImage(hudtexture,
+	draw2DImageFilterScaled(driver, hudtexture,
 			irr::core::rect<s32>(screensize.X/2, 0, screensize.X, screensize.Y),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, true);
 
@@ -380,19 +380,19 @@ void draw_top_bottom_3d_mode(Camera& camera, bool show_hud,
 	//makeColorKeyTexture mirrors texture so we do it twice to get it right again
 	driver->makeColorKeyTexture(hudtexture, irr::video::SColor(255, 0, 0, 0));
 
-	driver->draw2DImage(left_image,
+	draw2DImageFilterScaled(driver, left_image,
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y/2),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, false);
 
-	driver->draw2DImage(hudtexture,
+	draw2DImageFilterScaled(driver, hudtexture,
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y/2),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, true);
 
-	driver->draw2DImage(right_image,
+	draw2DImageFilterScaled(driver, right_image,
 			irr::core::rect<s32>(0, screensize.Y/2, screensize.X, screensize.Y),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, false);
 
-	driver->draw2DImage(hudtexture,
+	draw2DImageFilterScaled(driver, hudtexture,
 			irr::core::rect<s32>(0, screensize.Y/2, screensize.X, screensize.Y),
 			irr::core::rect<s32>(0, 0, screensize.X, screensize.Y), 0, 0, true);
 
@@ -416,84 +416,82 @@ void draw_plain(Camera& camera, bool show_hud, Hud& hud,
 		camera.drawWieldedTool();
 }
 
-void draw_scene(video::IVideoDriver* driver, scene::ISceneManager* smgr,
-		Camera& camera, Client& client, LocalPlayer* player, Hud& hud,
-		gui::IGUIEnvironment* guienv, std::vector<aabb3f> hilightboxes,
-		const v2u32& screensize, video::SColor skycolor, bool show_hud)
+void draw_scene(video::IVideoDriver *driver, scene::ISceneManager *smgr,
+		Camera &camera, Client& client, LocalPlayer *player, Hud &hud,
+		Mapper &mapper, gui::IGUIEnvironment *guienv,
+		std::vector<aabb3f> hilightboxes, const v2u32 &screensize,
+		video::SColor skycolor, bool show_hud, bool show_minimap)
 {
-	//TODO check if usefull
-	u32 scenetime = 0;
-	{
-		TimeTaker timer("smgr");
+	TimeTaker timer("smgr");
 
-		bool draw_wield_tool = (show_hud &&
-				(player->hud_flags & HUD_FLAG_WIELDITEM_VISIBLE) &&
-				camera.getCameraMode() < CAMERA_MODE_THIRD );
+	bool draw_wield_tool = (show_hud &&
+			(player->hud_flags & HUD_FLAG_WIELDITEM_VISIBLE) &&
+			camera.getCameraMode() < CAMERA_MODE_THIRD );
 
-		bool draw_crosshair = ((player->hud_flags & HUD_FLAG_CROSSHAIR_VISIBLE) &&
-				(camera.getCameraMode() != CAMERA_MODE_THIRD_FRONT));
+	bool draw_crosshair = ((player->hud_flags & HUD_FLAG_CROSSHAIR_VISIBLE) &&
+			(camera.getCameraMode() != CAMERA_MODE_THIRD_FRONT));
 
 #ifdef HAVE_TOUCHSCREENGUI
-		try {
-			draw_crosshair = !g_settings->getBool("touchtarget");
-		}
-		catch(SettingNotFoundException) {}
+	try {
+		draw_crosshair = !g_settings->getBool("touchtarget");
+	}
+	catch(SettingNotFoundException) {}
 #endif
 
-		std::string draw_mode = g_settings->get("3d_mode");
+	std::string draw_mode = g_settings->get("3d_mode");
 
-		smgr->drawAll();
+	smgr->drawAll();
 
-		if (draw_mode == "anaglyph")
-		{
-			draw_anaglyph_3d_mode(camera, show_hud, hud, hilightboxes, driver,
-					smgr, draw_wield_tool, client, guienv);
-			draw_crosshair = false;
-		}
-		else if (draw_mode == "interlaced")
-		{
-			draw_interlaced_3d_mode(camera, show_hud, hud, hilightboxes, driver,
-					smgr, screensize, draw_wield_tool, client, guienv, skycolor);
-			draw_crosshair = false;
-		}
-		else if (draw_mode == "sidebyside")
-		{
-			draw_sidebyside_3d_mode(camera, show_hud, hud, hilightboxes, driver,
-					smgr, screensize, draw_wield_tool, client, guienv, skycolor);
-			show_hud = false;
-		}
-		else if (draw_mode == "topbottom")
-		{
-			draw_top_bottom_3d_mode(camera, show_hud, hud, hilightboxes, driver,
-					smgr, screensize, draw_wield_tool, client, guienv, skycolor);
-			show_hud = false;
-		}
-		else {
-			draw_plain(camera, show_hud, hud, hilightboxes, driver,
-					draw_wield_tool, client, guienv);
-		}
-
-		/*
-			Post effects
-		*/
-		{
-			client.getEnv().getClientMap().renderPostFx(camera.getCameraMode());
-		}
-
-		//TODO how to make those 3d too
-		if (show_hud)
-		{
-			if (draw_crosshair)
-				hud.drawCrosshair();
-			hud.drawHotbar(client.getPlayerItem());
-			hud.drawLuaElements(camera.getOffset());
-		}
-
-		guienv->drawAll();
-
-		scenetime = timer.stop(true);
+	if (draw_mode == "anaglyph")
+	{
+		draw_anaglyph_3d_mode(camera, show_hud, hud, hilightboxes, driver,
+				smgr, draw_wield_tool, client, guienv);
+		draw_crosshair = false;
+	}
+	else if (draw_mode == "interlaced")
+	{
+		draw_interlaced_3d_mode(camera, show_hud, hud, hilightboxes, driver,
+				smgr, screensize, draw_wield_tool, client, guienv, skycolor);
+		draw_crosshair = false;
+	}
+	else if (draw_mode == "sidebyside")
+	{
+		draw_sidebyside_3d_mode(camera, show_hud, hud, hilightboxes, driver,
+				smgr, screensize, draw_wield_tool, client, guienv, skycolor);
+		show_hud = false;
+	}
+	else if (draw_mode == "topbottom")
+	{
+		draw_top_bottom_3d_mode(camera, show_hud, hud, hilightboxes, driver,
+				smgr, screensize, draw_wield_tool, client, guienv, skycolor);
+		show_hud = false;
+	}
+	else {
+		draw_plain(camera, show_hud, hud, hilightboxes, driver,
+				draw_wield_tool, client, guienv);
 	}
 
+	/*
+		Post effects
+	*/
+	{
+		client.getEnv().getClientMap().renderPostFx(camera.getCameraMode());
+	}
+
+	//TODO how to make those 3d too
+	if (show_hud)
+	{
+		if (draw_crosshair)
+			hud.drawCrosshair();
+		hud.drawHotbar(client.getPlayerItem());
+		hud.drawLuaElements(camera.getOffset());
+		if (show_minimap)
+			mapper.drawMinimap();
+	}
+
+	guienv->drawAll();
+
+	timer.stop(true);
 }
 
 /*
