@@ -22,15 +22,15 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "voxel.h"
 #include "noise.h"
+#include "mapgen.h"
 
 #define VMANIP_FLAG_DUNGEON_INSIDE VOXELFLAG_CHECKED1
 #define VMANIP_FLAG_DUNGEON_PRESERVE VOXELFLAG_CHECKED2
 #define VMANIP_FLAG_DUNGEON_UNTOUCHABLE (\
 		VMANIP_FLAG_DUNGEON_INSIDE|VMANIP_FLAG_DUNGEON_PRESERVE)
 
-class ManualMapVoxelManipulator;
+class MMVManip;
 class INodeDefManager;
-class Mapgen;
 
 v3s16 rand_ortho_dir(PseudoRandom &random, bool diagonal_dirs);
 v3s16 turn_xz(v3s16 olddir, int t);
@@ -44,7 +44,7 @@ struct DungeonParams {
 	content_t c_moss;
 	content_t c_stair;
 
-	int notifytype;
+	GenNotifyType notifytype;
 	bool diagonal_dirs;
 	float mossratio;
 	v3s16 holesize;
@@ -57,7 +57,7 @@ struct DungeonParams {
 
 class DungeonGen {
 public:
-	ManualMapVoxelManipulator *vm;
+	MMVManip *vm;
 	Mapgen *mg;
 	u32 blockseed;
 	PseudoRandom random;
@@ -65,14 +65,14 @@ public:
 
 	content_t c_torch;
 	DungeonParams dp;
-	
+
 	//RoomWalker
 	v3s16 m_pos;
 	v3s16 m_dir;
 
 	DungeonGen(Mapgen *mg, DungeonParams *dparams);
 	void generate(u32 bseed, v3s16 full_node_min, v3s16 full_node_max);
-	
+
 	void makeDungeon(v3s16 start_padding);
 	void makeRoom(v3s16 roomsize, v3s16 roomplace);
 	void makeCorridor(v3s16 doorplace, v3s16 doordir,
@@ -84,7 +84,7 @@ public:
 	bool findPlaceForDoor(v3s16 &result_place, v3s16 &result_dir);
 	bool findPlaceForRoomDoor(v3s16 roomsize, v3s16 &result_doorplace,
 			v3s16 &result_doordir, v3s16 &result_roomplace);
-			
+
 	void randomizeDir()
 	{
 		m_dir = rand_ortho_dir(random, dp.diagonal_dirs);
