@@ -31,7 +31,7 @@
 
 #include "intlGUIEditBox.h"
 
-#ifdef _IRR_COMPILE_WITH_GUI_
+#if defined(_IRR_COMPILE_WITH_GUI_) && IRRLICHT_VERSION_MAJOR == 1 && IRRLICHT_VERSION_MINOR < 9
 
 #include "IGUISkin.h"
 #include "IGUIEnvironment.h"
@@ -41,6 +41,7 @@
 //#include "irrlicht/os.cpp"
 #include "porting.h"
 //#include "Keycodes.h"
+#include "log.h"
 
 /*
 	todo:
@@ -270,7 +271,7 @@ bool intlGUIEditBox::OnEvent(const SEvent& event)
 			break;
 		case EET_KEY_INPUT_EVENT:
         {
-#if defined(linux)
+#if (defined(linux) || defined(__linux) || defined(__FreeBSD__))
             // ################################################################
 			// ValkaTR:
             // This part is the difference from the original intlGUIEditBox
