@@ -163,7 +163,7 @@ void ItemStack::deSerialize(std::istream &is, IItemDefManager *itemdef)
 	std::getline(is, tmp, ' ');
 	if(!tmp.empty())
 		throw SerializationError("Unexpected text after item name");
-	
+
 	if(name == "MaterialItem")
 	{
 		// Obsoleted on 2011-07-30
@@ -183,7 +183,8 @@ void ItemStack::deSerialize(std::istream &is, IItemDefManager *itemdef)
 		legacy_nimap.getName(material, name);
 		if(name == "")
 			name = "unknown_block";
-		name = itemdef->getAlias(name);
+		if (itemdef)
+			name = itemdef->getAlias(name);
 		count = materialcount;
 	}
 	else if(name == "MaterialItem2")
@@ -202,7 +203,8 @@ void ItemStack::deSerialize(std::istream &is, IItemDefManager *itemdef)
 		legacy_nimap.getName(material, name);
 		if(name == "")
 			name = "unknown_block";
-		name = itemdef->getAlias(name);
+		if (itemdef)
+			name = itemdef->getAlias(name);
 		count = materialcount;
 	}
 	else if(name == "node" || name == "NodeItem" || name == "MaterialItem3"
@@ -223,7 +225,8 @@ void ItemStack::deSerialize(std::istream &is, IItemDefManager *itemdef)
 			name = fnd.next(" ");
 		}
 		fnd.skip_over(" ");
-		name = itemdef->getAlias(name);
+		if (itemdef)
+			name = itemdef->getAlias(name);
 		count = stoi(trim(fnd.next("")));
 		if(count == 0)
 			count = 1;
@@ -252,7 +255,8 @@ void ItemStack::deSerialize(std::istream &is, IItemDefManager *itemdef)
 		count = 1;
 		// Then read wear
 		fnd.skip_over(" ");
-		name = itemdef->getAlias(name);
+		if (itemdef)
+			name = itemdef->getAlias(name);
 		wear = stoi(trim(fnd.next("")));
 	}
 	else
@@ -262,7 +266,8 @@ void ItemStack::deSerialize(std::istream &is, IItemDefManager *itemdef)
 			// The real thing
 
 			// Apply item aliases
-			name = itemdef->getAlias(name);
+			if (itemdef)
+				name = itemdef->getAlias(name);
 
 			// Read the count
 			std::string count_str;
@@ -294,9 +299,9 @@ void ItemStack::deSerialize(std::istream &is, IItemDefManager *itemdef)
 		} while(false);
 	}
 
-	if(name.empty() || count == 0)
+	if (name.empty() || count == 0)
 		clear();
-	else if(itemdef->get(name).type == ITEM_TOOL)
+	else if (itemdef && itemdef->get(name).type == ITEM_TOOL)
 		count = 1;
 }
 
@@ -308,12 +313,12 @@ void ItemStack::deSerialize(const std::string &str, IItemDefManager *itemdef)
 
 std::string ItemStack::getItemString() const
 {
-	// Get item string
 	std::ostringstream os(std::ios::binary);
 	serialize(os);
 	return os.str();
 }
 
+
 ItemStack ItemStack::addItem(const ItemStack &newitem_,
 		IItemDefManager *itemdef)
 {
@@ -473,7 +478,7 @@ void InventoryList::setName(const std::string &name)
 void InventoryList::serialize(std::ostream &os) const
 {
 	//os.imbue(std::locale("C"));
-	
+
 	os<<"Width "<<m_width<<"\n";
 
 	for(u32 i=0; i<m_items.size(); i++)
@@ -615,13 +620,13 @@ u32 InventoryList::getFreeSlots() const
 
 const ItemStack& InventoryList::getItem(u32 i) const
 {
-	assert(i < m_size);
+	assert(i < m_size); // Pre-condition
 	return m_items[i];
 }
 
 ItemStack& InventoryList::getItem(u32 i)
 {
-	assert(i < m_size);
+	assert(i < m_size); // Pre-condition
 	return m_items[i];
 }
 
@@ -638,7 +643,7 @@ ItemStack InventoryList::changeItem(u32 i, const ItemStack &newitem)
 
 void InventoryList::deleteItem(u32 i)
 {
-	assert(i < m_items.size());
+	assert(i < m_items.size()); // Pre-condition
 	m_items[i].clear();
 }
 
@@ -648,7 +653,7 @@ ItemStack InventoryList::addItem(const ItemStack &newitem_)
 
 	if(newitem.empty())
 		return newitem;
-	
+
 	/*
 		First try to find if it could be added to some existing items
 	*/
@@ -777,10 +782,46 @@ ItemStack InventoryList::peekItem(u32 i, u32 peekcount) const
 	return m_items[i].peekItem(peekcount);
 }
 
-void InventoryList::moveItem(u32 i, InventoryList *dest, u32 dest_i, u32 count)
+void InventoryList::moveItemSomewhere(u32 i, InventoryList *dest, u32 count)
+{
+	// Take item from source list
+	ItemStack item1;
+	if (count == 0)
+		item1 = changeItem(i, ItemStack());
+	else
+		item1 = takeItem(i, count);
+
+	if (item1.empty())
+		return;
+
+	// Try to add the item to destination list
+	u32 dest_size = dest->getSize();
+	// First try all the non-empty slots
+	for (u32 dest_i = 0; dest_i < dest_size; dest_i++) {
+		if (!m_items[dest_i].empty()) {
+			item1 = dest->addItem(dest_i, item1);
+			if (item1.empty()) return;
+		}
+	}
+
+	// Then try all the empty ones
+	for (u32 dest_i = 0; dest_i < dest_size; dest_i++) {
+		if (m_items[dest_i].empty()) {
+			item1 = dest->addItem(dest_i, item1);
+			if (item1.empty()) return;
+		}
+	}
+
+	// If we reach this, the item was not fully added
+	// Add the remaining part back to the source item
+	addItem(i, item1);
+}
+
+u32 InventoryList::moveItem(u32 i, InventoryList *dest, u32 dest_i,
+		u32 count, bool swap_if_needed, bool *did_swap)
 {
 	if(this == dest && i == dest_i)
-		return;
+		return count;
 
 	// Take item from source list
 	ItemStack item1;
@@ -790,7 +831,7 @@ void InventoryList::moveItem(u32 i, InventoryList *dest, u32 dest_i, u32 count)
 		item1 = takeItem(i, count);
 
 	if(item1.empty())
-		return;
+		return 0;
 
 	// Try to add the item to destination list
 	u32 oldcount = item1.count;
@@ -808,8 +849,11 @@ void InventoryList::moveItem(u32 i, InventoryList *dest, u32 dest_i, u32 count)
 
 		// If olditem is returned, nothing was added.
 		// Swap the items
-		if(nothing_added)
-		{
+		if (nothing_added && swap_if_needed) {
+			// Tell that we swapped
+			if (did_swap != NULL) {
+				*did_swap = true;
+			}
 			// Take item from source list
 			item1 = changeItem(i, ItemStack());
 			// Adding was not possible, swap the items.
@@ -818,6 +862,7 @@ void InventoryList::moveItem(u32 i, InventoryList *dest, u32 dest_i, u32 count)
 			changeItem(i, item2);
 		}
 	}
+	return (oldcount - item1.count);
 }
 
 /*
@@ -831,6 +876,7 @@ Inventory::~Inventory()
 
 void Inventory::clear()
 {
+	m_dirty = true;
 	for(u32 i=0; i<m_lists.size(); i++)
 	{
 		delete m_lists[i];
@@ -840,6 +886,7 @@ void Inventory::clear()
 
 void Inventory::clearContents()
 {
+	m_dirty = true;
 	for(u32 i=0; i<m_lists.size(); i++)
 	{
 		InventoryList *list = m_lists[i];
@@ -852,12 +899,14 @@ void Inventory::clearContents()
 
 Inventory::Inventory(IItemDefManager *itemdef)
 {
+	m_dirty = false;
 	m_itemdef = itemdef;
 }
 
 Inventory::Inventory(const Inventory &other)
 {
 	*this = other;
+	m_dirty = false;
 }
 
 Inventory & Inventory::operator = (const Inventory &other)
@@ -865,6 +914,7 @@ Inventory & Inventory::operator = (const Inventory &other)
 	// Gracefully handle self assignment
 	if(this != &other)
 	{
+		m_dirty = true;
 		clear();
 		m_itemdef = other.m_itemdef;
 		for(u32 i=0; i<other.m_lists.size(); i++)
@@ -938,13 +988,14 @@ void Inventory::deSerialize(std::istream &is)
 		}
 		else
 		{
-			throw SerializationError("invalid inventory specifier");
+			throw SerializationError("invalid inventory specifier: " + name);
 		}
 	}
 }
 
 InventoryList * Inventory::addList(const std::string &name, u32 size)
 {
+	m_dirty = true;
 	s32 i = getListIndex(name);
 	if(i != -1)
 	{
@@ -990,6 +1041,7 @@ bool Inventory::deleteList(const std::string &name)
 	s32 i = getListIndex(name);
 	if(i == -1)
 		return false;
+	m_dirty = true;
 	delete m_lists[i];
 	m_lists.erase(m_lists.begin() + i);
 	return true;
