@@ -20,9 +20,6 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #ifndef IRRLICHTTYPES_EXTRABLOATED_HEADER
 #define IRRLICHTTYPES_EXTRABLOATED_HEADER
 
-#define endSceneX(d){d->draw2DLine(v2s32(0,0),v2s32(1,0),\
-video::SColor(255,30,30,30));d->endScene();}
-
 #include "irrlichttypes_bloated.h"
 
 #ifndef SERVER
