@@ -18,12 +18,11 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 */
 
 #include "keycode.h"
-#include "main.h" // For g_settings
 #include "exceptions.h"
 #include "settings.h"
 #include "log.h"
-#include "hex.h"
 #include "debug.h"
+#include "util/hex.h"
 
 class UnknownKeycode : public BaseException
 {
@@ -257,13 +256,18 @@ KeyPress::KeyPress() :
 
 KeyPress::KeyPress(const char *name)
 {
-	if (strlen(name) > 4) {
+	if (name[0] == 0) {
+		Key = irr::KEY_KEY_CODES_COUNT;
+		Char = L'\0';
+		return;
+	} else if (strlen(name) > 4) {
 		try {
 			Key = keyname_to_keycode(name);
 			m_name = name;
 			if (strlen(name) > 8 && strncmp(name, "KEY_KEY_", 8) == 0) {
 				int chars_read = mbtowc(&Char, name + 8, 1);
-				assert (chars_read == 1 && "unexpected multibyte character");
+
+				FATAL_ERROR_IF(chars_read != 1, "Unexpected multibyte character");
 			} else
 				Char = L'\0';
 			return;
@@ -275,7 +279,8 @@ KeyPress::KeyPress(const char *name)
 		try {
 			Key = keyname_to_keycode(m_name.c_str());
 			int chars_read = mbtowc(&Char, name, 1);
-			assert (chars_read == 1 && "unexpected multibyte character");
+
+			FATAL_ERROR_IF(chars_read != 1, "Unexpected multibyte character");
 			return;
 		} catch (UnknownKeycode &e) {};
 	}
@@ -285,7 +290,7 @@ KeyPress::KeyPress(const char *name)
 	Key = irr::KEY_KEY_CODES_COUNT;
 
 	int mbtowc_ret = mbtowc(&Char, name, 1);
-	assert (mbtowc_ret == 1 && "unexpected multibyte character");
+	FATAL_ERROR_IF(mbtowc_ret != 1, "Unexpected multibyte character");
 	m_name = name[0];
 }
 
