@@ -48,7 +48,7 @@ public:
 	
 	void move(f32 dtime, Environment *env, f32 pos_max_d);
 	void move(f32 dtime, Environment *env, f32 pos_max_d,
-			std::list<CollisionInfo> *collision_info);
+			std::vector<CollisionInfo> *collision_info);
 
 	void applyControl(float dtime);
 
@@ -62,8 +62,6 @@ public:
 	unsigned int last_keyPressed;
 
 	float camera_impact;
-	v3f eye_offset_first;
-	v3f eye_offset_third;
 
 	int last_animation;
 	float last_animation_speed;
@@ -78,7 +76,7 @@ public:
 	}
 
 	void setCAO(GenericCAO* toset) {
-		assert( m_cao == NULL );
+		assert( m_cao == NULL ); // Pre-condition
 		m_cao = toset;
 	}
 
@@ -87,12 +85,15 @@ private:
 	v3s16 m_sneak_node;
 	// Whether the player is allowed to sneak
 	bool m_sneak_node_exists;
+	// Whether recalculation of the sneak node is needed
+	bool m_need_to_get_new_sneak_node;
+	// Stores the max player uplift by m_sneak_node and is updated
+	// when m_need_to_get_new_sneak_node == true
+	f32 m_sneak_node_bb_ymax;
 	// Node below player, used to determine whether it has been removed,
 	// and its old type
 	v3s16 m_old_node_below;
 	std::string m_old_node_below_type;
-	// Whether recalculation of the sneak node is needed
-	bool m_need_to_get_new_sneak_node;
 	bool m_can_jump;
 
 	GenericCAO* m_cao;
