@@ -21,16 +21,17 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "light.h"
 #include "mapblock.h"
 #include "map.h"
-#include "main.h" // for g_profiler
 #include "profiler.h"
 #include "nodedef.h"
 #include "gamedef.h"
 #include "mesh.h"
+#include "minimap.h"
 #include "content_mapblock.h"
 #include "noise.h"
 #include "shader.h"
 #include "settings.h"
 #include "util/directiontables.h"
+#include <IMeshManipulator.h>
 
 static void applyFacesShading(video::SColor& color, float factor)
 {
@@ -42,7 +43,7 @@ static void applyFacesShading(video::SColor& color, float factor)
 	MeshMakeData
 */
 
-MeshMakeData::MeshMakeData(IGameDef *gamedef):
+MeshMakeData::MeshMakeData(IGameDef *gamedef, bool use_shaders):
 	m_vmanip(),
 	m_blockpos(-1337,-1337,-1337),
 	m_crack_pos_relative(-1337, -1337, -1337),
@@ -50,7 +51,8 @@ MeshMakeData::MeshMakeData(IGameDef *gamedef):
 	m_smooth_lighting(false),
 	m_show_hud(false),
 	m_highlight_mesh_color(255, 255, 255, 255),
-	m_gamedef(gamedef)
+	m_gamedef(gamedef),
+	m_use_shaders(use_shaders)
 {}
 
 void MeshMakeData::fill(MapBlock *block)
@@ -247,7 +249,7 @@ static u16 getSmoothLightCombined(v3s16 p, MeshMakeData *data)
 
 	for (u32 i = 0; i < 8; i++)
 	{
-		MapNode n = data->m_vmanip.getNodeNoEx(p - dirs8[i]);
+		const MapNode &n = data->m_vmanip.getNodeRefUnsafeCheckFlags(p - dirs8[i]);
 
 		// if it's CONTENT_IGNORE we can't do any light calculations
 		if (n.getContent() == CONTENT_IGNORE) {
@@ -437,8 +439,6 @@ struct FastFace
 static void makeFastFace(TileSpec tile, u16 li0, u16 li1, u16 li2, u16 li3,
 		v3f p, v3s16 dir, v3f scale, u8 light_source, std::vector<FastFace> &dest)
 {
-	FastFace face;
-
 	// Position is at the center of the cube.
 	v3f pos = p * BS;
 
@@ -589,6 +589,10 @@ static void makeFastFace(TileSpec tile, u16 li0, u16 li1, u16 li2, u16 li3,
 
 	u8 alpha = tile.alpha;
 
+	dest.push_back(FastFace());
+
+	FastFace& face = *dest.rbegin();
+
 	face.vertices[0] = video::S3DVertex(vertex_pos[0], normal,
 			MapBlock_LightColor(alpha, li0, light_source),
 			core::vector2d<f32>(x0+w*abs_scale, y0+h));
@@ -603,7 +607,6 @@ static void makeFastFace(TileSpec tile, u16 li0, u16 li1, u16 li2, u16 li3,
 			core::vector2d<f32>(x0+w*abs_scale, y0));
 
 	face.tile = tile;
-	dest.push_back(face);
 }
 
 /*
@@ -744,8 +747,8 @@ TileSpec getNodeTile(MapNode mn, v3s16 p, v3s16 dir, MeshMakeData *data)
 static void getTileInfo(
 		// Input:
 		MeshMakeData *data,
-		v3s16 p,
-		v3s16 face_dir,
+		const v3s16 &p,
+		const v3s16 &face_dir,
 		// Output:
 		bool &makes_face,
 		v3s16 &p_corrected,
@@ -759,14 +762,20 @@ static void getTileInfo(
 	INodeDefManager *ndef = data->m_gamedef->ndef();
 	v3s16 blockpos_nodes = data->m_blockpos * MAP_BLOCKSIZE;
 
-	MapNode n0 = vmanip.getNodeNoEx(blockpos_nodes + p);
+	MapNode &n0 = vmanip.getNodeRefUnsafe(blockpos_nodes + p);
 
 	// Don't even try to get n1 if n0 is already CONTENT_IGNORE
-	if (n0.getContent() == CONTENT_IGNORE ) {
+	if (n0.getContent() == CONTENT_IGNORE) {
+		makes_face = false;
+		return;
+	}
+
+	const MapNode &n1 = vmanip.getNodeRefUnsafeCheckFlags(blockpos_nodes + p + face_dir);
+
+	if (n1.getContent() == CONTENT_IGNORE) {
 		makes_face = false;
 		return;
 	}
-	MapNode n1 = vmanip.getNodeNoEx(blockpos_nodes + p + face_dir);
 
 	// This is hackish
 	bool equivalent = false;
@@ -1020,7 +1029,10 @@ static void updateAllFastFaceRows(MeshMakeData *data,
 
 MapBlockMesh::MapBlockMesh(MeshMakeData *data, v3s16 camera_offset):
 	m_mesh(new scene::SMesh()),
+	m_minimap_mapblock(NULL),
 	m_gamedef(data->m_gamedef),
+	m_tsrc(m_gamedef->getTextureSource()),
+	m_shdrsrc(m_gamedef->getShaderSource()),
 	m_animation_force_timer(0), // force initial animation
 	m_last_crack(-1),
 	m_crack_materials(),
@@ -1028,14 +1040,21 @@ MapBlockMesh::MapBlockMesh(MeshMakeData *data, v3s16 camera_offset):
 	m_last_daynight_ratio((u32) -1),
 	m_daynight_diffs()
 {
-	m_enable_shaders = g_settings->getBool("enable_shaders");
+	m_enable_shaders = data->m_use_shaders;
 	m_enable_highlighting = g_settings->getBool("enable_node_highlighting");
 
+	if (g_settings->getBool("enable_minimap")) {
+		m_minimap_mapblock = new MinimapMapblock;
+		m_minimap_mapblock->getMinimapNodes(
+			&data->m_vmanip, data->m_blockpos * MAP_BLOCKSIZE);
+	}
+
 	// 4-21ms for MAP_BLOCKSIZE=16  (NOTE: probably outdated)
 	// 24-155ms for MAP_BLOCKSIZE=32  (NOTE: probably outdated)
 	//TimeTaker timer1("MapBlockMesh()");
 
 	std::vector<FastFace> fastfaces_new;
+	fastfaces_new.reserve(512);
 
 	/*
 		We are including the faces of the trailing edges of the block.
@@ -1102,8 +1121,6 @@ MapBlockMesh::MapBlockMesh(MeshMakeData *data, v3s16 camera_offset):
 	/*
 		Convert MeshCollector to SMesh
 	*/
-	ITextureSource *tsrc = m_gamedef->tsrc();
-	IShaderSource *shdrsrc = m_gamedef->getShaderSource();
 
 	for(u32 i = 0; i < collector.prebuffers.size(); i++)
 	{
@@ -1115,13 +1132,13 @@ MapBlockMesh::MapBlockMesh(MeshMakeData *data, v3s16 camera_offset):
 		{
 			// Find the texture name plus ^[crack:N:
 			std::ostringstream os(std::ios::binary);
-			os<<tsrc->getTextureName(p.tile.texture_id)<<"^[crack";
+			os<<m_tsrc->getTextureName(p.tile.texture_id)<<"^[crack";
 			if(p.tile.material_flags & MATERIAL_FLAG_CRACK_OVERLAY)
 				os<<"o";  // use ^[cracko
 			os<<":"<<(u32)p.tile.animation_frame_count<<":";
 			m_crack_materials.insert(std::make_pair(i, os.str()));
 			// Replace tile texture with the cracked one
-			p.tile.texture = tsrc->getTexture(
+			p.tile.texture = m_tsrc->getTextureForMesh(
 					os.str()+"0",
 					&p.tile.texture_id);
 		}
@@ -1146,24 +1163,25 @@ MapBlockMesh::MapBlockMesh(MeshMakeData *data, v3s16 camera_offset):
 		}
 
 		if(m_enable_highlighting && p.tile.material_flags & MATERIAL_FLAG_HIGHLIGHTED)
-			m_highlighted_materials.push_back(i);	
+			m_highlighted_materials.push_back(i);
 
 		for(u32 j = 0; j < p.vertices.size(); j++)
 		{
+			video::S3DVertexTangents *vertex = &p.vertices[j];
 			// Note applyFacesShading second parameter is precalculated sqrt
 			// value for speed improvement
 			// Skip it for lightsources and top faces.
-			video::SColor &vc = p.vertices[j].Color;
+			video::SColor &vc = vertex->Color;
 			if (!vc.getBlue()) {
-				if (p.vertices[j].Normal.Y < -0.5) {
+				if (vertex->Normal.Y < -0.5) {
 					applyFacesShading (vc, 0.447213);
-				} else if (p.vertices[j].Normal.X > 0.5) {
+				} else if (vertex->Normal.X > 0.5) {
 					applyFacesShading (vc, 0.670820);
-				} else if (p.vertices[j].Normal.X < -0.5) {
+				} else if (vertex->Normal.X < -0.5) {
 					applyFacesShading (vc, 0.670820);
-				} else if (p.vertices[j].Normal.Z > 0.5) {
+				} else if (vertex->Normal.Z > 0.5) {
 					applyFacesShading (vc, 0.836660);
-				} else if (p.vertices[j].Normal.Z < -0.5) {
+				} else if (vertex->Normal.Z < -0.5) {
 					applyFacesShading (vc, 0.836660);
 				}
 			}
@@ -1191,33 +1209,28 @@ MapBlockMesh::MapBlockMesh(MeshMakeData *data, v3s16 camera_offset):
 			material.MaterialType = video::EMT_TRANSPARENT_ADD_COLOR;
 		} else {
 			if (m_enable_shaders) {
-				material.MaterialType = shdrsrc->getShaderInfo(p.tile.shader_id).material;
+				material.MaterialType = m_shdrsrc->getShaderInfo(p.tile.shader_id).material;
 				p.tile.applyMaterialOptionsWithShaders(material);
 				if (p.tile.normal_texture) {
 					material.setTexture(1, p.tile.normal_texture);
-					material.setTexture(2, tsrc->getTexture("enable_img.png"));
-				} else {
-					material.setTexture(2, tsrc->getTexture("disable_img.png"));
 				}
+				material.setTexture(2, p.tile.flags_texture);
 			} else {
 				p.tile.applyMaterialOptions(material);
 			}
 		}
 
-		// Create meshbuffer
-		// This is a "Standard MeshBuffer",
-		// it's a typedeffed CMeshBuffer<video::S3DVertex>
-		scene::SMeshBuffer *buf = new scene::SMeshBuffer();
-		// Set material
-		buf->Material = material;
-		// Add to mesh
-		m_mesh->addMeshBuffer(buf);
-		// Mesh grabbed it
-		buf->drop();
-		buf->append(&p.vertices[0], p.vertices.size(),
-				&p.indices[0], p.indices.size());
-	}
-
+	// Create meshbuffer
+	scene::SMeshBufferTangents *buf = new scene::SMeshBufferTangents();
+	// Set material
+	buf->Material = material;
+	// Add to mesh
+	m_mesh->addMeshBuffer(buf);
+	// Mesh grabbed it
+	buf->drop();
+	buf->append(&p.vertices[0], p.vertices.size(),
+		&p.indices[0], p.indices.size());
+}
 	m_camera_offset = camera_offset;
 
 	/*
@@ -1226,6 +1239,11 @@ MapBlockMesh::MapBlockMesh(MeshMakeData *data, v3s16 camera_offset):
 
 	translateMesh(m_mesh, intToFloat(data->m_blockpos * MAP_BLOCKSIZE - camera_offset, BS));
 
+	if (m_enable_shaders) {
+		scene::IMeshManipulator* meshmanip = m_gamedef->getSceneManager()->getMeshManipulator();
+		meshmanip->recalculateTangents(m_mesh, true, false, false);
+	}
+
 	if(m_mesh)
 	{
 #if 0
@@ -1260,11 +1278,11 @@ MapBlockMesh::~MapBlockMesh()
 {
 	m_mesh->drop();
 	m_mesh = NULL;
+	delete m_minimap_mapblock;
 }
 
 bool MapBlockMesh::animate(bool faraway, float time, int crack, u32 daynight_ratio)
 {
-
 	if(!m_has_animation)
 	{
 		m_animation_force_timer = 100000;
@@ -1284,12 +1302,11 @@ bool MapBlockMesh::animate(bool faraway, float time, int crack, u32 daynight_rat
 			std::string basename = i->second;
 
 			// Create new texture name from original
-			ITextureSource *tsrc = m_gamedef->getTextureSource();
 			std::ostringstream os;
 			os<<basename<<crack;
 			u32 new_texture_id = 0;
 			video::ITexture *new_texture =
-				tsrc->getTexture(os.str(), &new_texture_id);
+				m_tsrc->getTextureForMesh(os.str(), &new_texture_id);
 			buf->getMaterial().setTexture(0, new_texture);
 
 			// If the current material is also animated,
@@ -1325,17 +1342,14 @@ bool MapBlockMesh::animate(bool faraway, float time, int crack, u32 daynight_rat
 		m_animation_frames[i->first] = frame;
 
 		scene::IMeshBuffer *buf = m_mesh->getMeshBuffer(i->first);
-		ITextureSource *tsrc = m_gamedef->getTextureSource();
 
 		FrameSpec animation_frame = tile.frames[frame];
 		buf->getMaterial().setTexture(0, animation_frame.texture);
 		if (m_enable_shaders) {
 			if (animation_frame.normal_texture) {
 				buf->getMaterial().setTexture(1, animation_frame.normal_texture);
-				buf->getMaterial().setTexture(2, tsrc->getTexture("enable_img.png"));
-			} else {
-				buf->getMaterial().setTexture(2, tsrc->getTexture("disable_img.png"));
 			}
+			buf->getMaterial().setTexture(2, animation_frame.flags_texture);
 		}
 	}
 
@@ -1347,16 +1361,14 @@ bool MapBlockMesh::animate(bool faraway, float time, int crack, u32 daynight_rat
 				i != m_daynight_diffs.end(); i++)
 		{
 			scene::IMeshBuffer *buf = m_mesh->getMeshBuffer(i->first);
-			video::S3DVertex *vertices = (video::S3DVertex*)buf->getVertices();
+			video::S3DVertexTangents *vertices = (video::S3DVertexTangents *)buf->getVertices();
 			for(std::map<u32, std::pair<u8, u8 > >::iterator
 					j = i->second.begin();
 					j != i->second.end(); j++)
 			{
-				u32 vertexIndex = j->first;
 				u8 day = j->second.first;
 				u8 night = j->second.second;
-				finalColorBlend(vertices[vertexIndex].Color,
-						day, night, daynight_ratio);
+				finalColorBlend(vertices[j->first].Color, day, night, daynight_ratio);
 			}
 		}
 		m_last_daynight_ratio = daynight_ratio;
@@ -1365,7 +1377,7 @@ bool MapBlockMesh::animate(bool faraway, float time, int crack, u32 daynight_rat
 	// Node highlighting
 	if (m_enable_highlighting) {
 		u8 day = m_highlight_mesh_color.getRed();
-		u8 night = m_highlight_mesh_color.getGreen();	
+		u8 night = m_highlight_mesh_color.getGreen();
 		video::SColor hc;
 		finalColorBlend(hc, day, night, daynight_ratio);
 		float sin_r = 0.07 * sin(1.5 * time);
@@ -1380,7 +1392,7 @@ bool MapBlockMesh::animate(bool faraway, float time, int crack, u32 daynight_rat
 			i != m_highlighted_materials.end(); i++)
 		{
 			scene::IMeshBuffer *buf = m_mesh->getMeshBuffer(*i);
-			video::S3DVertex *vertices = (video::S3DVertex*)buf->getVertices();
+			video::S3DVertexTangents *vertices = (video::S3DVertexTangents*)buf->getVertices();
 			for (u32 j = 0; j < buf->getVertexCount() ;j++)
 				vertices[j].Color = hc;
 		}
@@ -1405,42 +1417,40 @@ void MeshCollector::append(const TileSpec &tile,
 		const video::S3DVertex *vertices, u32 numVertices,
 		const u16 *indices, u32 numIndices)
 {
-	if(numIndices > 65535)
-	{
+	if (numIndices > 65535) {
 		dstream<<"FIXME: MeshCollector::append() called with numIndices="<<numIndices<<" (limit 65535)"<<std::endl;
 		return;
 	}
 
 	PreMeshBuffer *p = NULL;
-	for(u32 i=0; i<prebuffers.size(); i++)
-	{
+	for (u32 i = 0; i < prebuffers.size(); i++) {
 		PreMeshBuffer &pp = prebuffers[i];
-		if(pp.tile != tile)
+		if (pp.tile != tile)
 			continue;
-		if(pp.indices.size() + numIndices > 65535)
+		if (pp.indices.size() + numIndices > 65535)
 			continue;
 
 		p = &pp;
 		break;
 	}
 
-	if(p == NULL)
-	{
+	if (p == NULL) {
 		PreMeshBuffer pp;
 		pp.tile = tile;
 		prebuffers.push_back(pp);
-		p = &prebuffers[prebuffers.size()-1];
+		p = &prebuffers[prebuffers.size() - 1];
 	}
 
 	u32 vertex_count = p->vertices.size();
-	for(u32 i=0; i<numIndices; i++)
-	{
+	for (u32 i = 0; i < numIndices; i++)	{
 		u32 j = indices[i] + vertex_count;
 		p->indices.push_back(j);
 	}
-	for(u32 i=0; i<numVertices; i++)
-	{
-		p->vertices.push_back(vertices[i]);
+
+	for (u32 i = 0; i < numVertices; i++) {
+		video::S3DVertexTangents vert(vertices[i].Pos, vertices[i].Normal,
+			vertices[i].Color, vertices[i].TCoords);
+		p->vertices.push_back(vert);
 	}
 }
 
@@ -1453,15 +1463,13 @@ void MeshCollector::append(const TileSpec &tile,
 		const u16 *indices, u32 numIndices,
 		v3f pos, video::SColor c)
 {
-	if(numIndices > 65535)
-	{
+	if (numIndices > 65535) {
 		dstream<<"FIXME: MeshCollector::append() called with numIndices="<<numIndices<<" (limit 65535)"<<std::endl;
 		return;
 	}
 
 	PreMeshBuffer *p = NULL;
-	for(u32 i=0; i<prebuffers.size(); i++)
-	{
+	for (u32 i = 0; i < prebuffers.size(); i++) {
 		PreMeshBuffer &pp = prebuffers[i];
 		if(pp.tile != tile)
 			continue;
@@ -1472,25 +1480,22 @@ void MeshCollector::append(const TileSpec &tile,
 		break;
 	}
 
-	if(p == NULL)
-	{
+	if (p == NULL) {
 		PreMeshBuffer pp;
 		pp.tile = tile;
 		prebuffers.push_back(pp);
-		p = &prebuffers[prebuffers.size()-1];
+		p = &prebuffers[prebuffers.size() - 1];
 	}
 
 	u32 vertex_count = p->vertices.size();
-	for(u32 i=0; i<numIndices; i++)
-	{
+	for (u32 i = 0; i < numIndices; i++) {
 		u32 j = indices[i] + vertex_count;
 		p->indices.push_back(j);
 	}
-	for(u32 i=0; i<numVertices; i++)
-	{
-		video::S3DVertex vert = vertices[i];
-		vert.Pos += pos;
-		vert.Color = c;		
+
+	for (u32 i = 0; i < numVertices; i++) {
+		video::S3DVertexTangents vert(vertices[i].Pos + pos, vertices[i].Normal,
+			c, vertices[i].TCoords);
 		p->vertices.push_back(vert);
 	}
 }
