@@ -27,9 +27,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "content_sao.h"
 #include "nodedef.h"
 #include "voxelalgorithms.h"
-#include "profiler.h"
 #include "settings.h" // For g_settings
-#include "main.h" // For g_profiler
 #include "emerge.h"
 #include "dungeongen.h"
 #include "cavegen.h"
@@ -42,7 +40,6 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 
 FlagDesc flagdesc_mapgen_v5[] = {
-	{"blobs", MGV5_BLOBS},
 	{NULL,         0}
 };
 
@@ -60,6 +57,8 @@ MapgenV5::MapgenV5(int mapgenid, MapgenParams *params, EmergeManager *emerge)
 
 	this->biomemap  = new u8[csize.X * csize.Z];
 	this->heightmap = new s16[csize.X * csize.Z];
+	this->heatmap   = NULL;
+	this->humidmap  = NULL;
 
 	MapgenV5Params *sp = (MapgenV5Params *)params->sparams;
 	this->spflags      = sp->spflags;
@@ -70,42 +69,42 @@ MapgenV5::MapgenV5(int mapgenid, MapgenParams *params, EmergeManager *emerge)
 	noise_height       = new Noise(&sp->np_height,       seed, csize.X, csize.Z);
 
 	// 3D terrain noise
-	noise_cave1        = new Noise(&sp->np_cave1,   seed, csize.X, csize.Y + 2, csize.Z);
-	noise_cave2        = new Noise(&sp->np_cave2,   seed, csize.X, csize.Y + 2, csize.Z);
-	noise_ground       = new Noise(&sp->np_ground,  seed, csize.X, csize.Y + 2, csize.Z);
-	noise_crumble      = new Noise(&sp->np_crumble, seed, csize.X, csize.Y + 2, csize.Z);
-	noise_wetness      = new Noise(&sp->np_wetness, seed, csize.X, csize.Y + 2, csize.Z);
+	noise_cave1  = new Noise(&sp->np_cave1,  seed, csize.X, csize.Y + 2, csize.Z);
+	noise_cave2  = new Noise(&sp->np_cave2,  seed, csize.X, csize.Y + 2, csize.Z);
+	noise_ground = new Noise(&sp->np_ground, seed, csize.X, csize.Y + 2, csize.Z);
 
 	// Biome noise
-	noise_heat         = new Noise(&params->np_biome_heat,     seed, csize.X, csize.Z);
-	noise_humidity     = new Noise(&params->np_biome_humidity, seed, csize.X, csize.Z);
+	noise_heat           = new Noise(&params->np_biome_heat,           seed, csize.X, csize.Z);
+	noise_humidity       = new Noise(&params->np_biome_humidity,       seed, csize.X, csize.Z);
+	noise_heat_blend     = new Noise(&params->np_biome_heat_blend,     seed, csize.X, csize.Z);
+	noise_humidity_blend = new Noise(&params->np_biome_humidity_blend, seed, csize.X, csize.Z);
 
 	//// Resolve nodes to be used
 	INodeDefManager *ndef = emerge->ndef;
 
-	c_stone           = ndef->getId("mapgen_stone");
-	c_dirt            = ndef->getId("mapgen_dirt");
-	c_dirt_with_grass = ndef->getId("mapgen_dirt_with_grass");
-	c_sand            = ndef->getId("mapgen_sand");
-	c_water_source    = ndef->getId("mapgen_water_source");
-	c_lava_source     = ndef->getId("mapgen_lava_source");
-	c_gravel          = ndef->getId("mapgen_gravel");
-	c_cobble          = ndef->getId("mapgen_cobble");
-	c_ice             = ndef->getId("default:ice");
-	c_mossycobble     = ndef->getId("mapgen_mossycobble");
-	c_sandbrick       = ndef->getId("mapgen_sandstonebrick");
-	c_stair_cobble    = ndef->getId("mapgen_stair_cobble");
-	c_stair_sandstone = ndef->getId("mapgen_stair_sandstone");
+	c_stone                = ndef->getId("mapgen_stone");
+	c_water_source         = ndef->getId("mapgen_water_source");
+	c_lava_source          = ndef->getId("mapgen_lava_source");
+	c_desert_stone         = ndef->getId("mapgen_desert_stone");
+	c_ice                  = ndef->getId("mapgen_ice");
+	c_sandstone            = ndef->getId("mapgen_sandstone");
+
+	c_cobble               = ndef->getId("mapgen_cobble");
+	c_stair_cobble         = ndef->getId("mapgen_stair_cobble");
+	c_mossycobble          = ndef->getId("mapgen_mossycobble");
+	c_sandstonebrick       = ndef->getId("mapgen_sandstonebrick");
+	c_stair_sandstonebrick = ndef->getId("mapgen_stair_sandstonebrick");
+
 	if (c_ice == CONTENT_IGNORE)
 		c_ice = CONTENT_AIR;
 	if (c_mossycobble == CONTENT_IGNORE)
 		c_mossycobble = c_cobble;
-	if (c_sandbrick == CONTENT_IGNORE)
-		c_sandbrick = c_desert_stone;
 	if (c_stair_cobble == CONTENT_IGNORE)
 		c_stair_cobble = c_cobble;
-	if (c_stair_sandstone == CONTENT_IGNORE)
-		c_stair_sandstone = c_sandbrick;
+	if (c_sandstonebrick == CONTENT_IGNORE)
+		c_sandstonebrick = c_sandstone;
+	if (c_stair_sandstonebrick == CONTENT_IGNORE)
+		c_stair_sandstonebrick = c_sandstone;
 }
 
 
@@ -117,11 +116,11 @@ MapgenV5::~MapgenV5()
 	delete noise_cave1;
 	delete noise_cave2;
 	delete noise_ground;
-	delete noise_crumble;
-	delete noise_wetness;
 
 	delete noise_heat;
 	delete noise_humidity;
+	delete noise_heat_blend;
+	delete noise_humidity_blend;
 
 	delete[] heightmap;
 	delete[] biomemap;
@@ -130,7 +129,7 @@ MapgenV5::~MapgenV5()
 
 MapgenV5Params::MapgenV5Params()
 {
-	spflags = MGV5_BLOBS;
+	spflags = 0;
 
 	np_filler_depth = NoiseParams(0, 1,  v3f(150, 150, 150), 261,    4, 0.7,  2.0);
 	np_factor       = NoiseParams(0, 1,  v3f(250, 250, 250), 920381, 3, 0.45, 2.0);
@@ -138,21 +137,14 @@ MapgenV5Params::MapgenV5Params()
 	np_cave1        = NoiseParams(0, 12, v3f(50,  50,  50),  52534,  4, 0.5,  2.0);
 	np_cave2        = NoiseParams(0, 12, v3f(50,  50,  50),  10325,  4, 0.5,  2.0);
 	np_ground       = NoiseParams(0, 40, v3f(80,  80,  80),  983240, 4, 0.55, 2.0, NOISE_FLAG_EASED);
-	np_crumble      = NoiseParams(0, 1,  v3f(20,  20,  20),  34413,  3, 1.3,  2.0, NOISE_FLAG_EASED);
-	np_wetness      = NoiseParams(0, 1,  v3f(40,  40,  40),  32474,  4, 1.1,  2.0);
 }
 
 
-// Scaling the output of the noise function affects the overdrive of the
-// contour function, which affects the shape of the output considerably.
-
-// Two original MT 0.3 parameters for non-eased noise:
-
 //#define CAVE_NOISE_SCALE 12.0
-//#define CAVE_NOISE_THRESHOLD (1.5/CAVE_NOISE_SCALE)
+//#define CAVE_NOISE_THRESHOLD (1.5/CAVE_NOISE_SCALE) = 0.125
 
 
-void MapgenV5Params::readParams(Settings *settings)
+void MapgenV5Params::readParams(const Settings *settings)
 {
 	settings->getFlagStrNoEx("mgv5_spflags", spflags, flagdesc_mapgen_v5);
 
@@ -162,12 +154,10 @@ void MapgenV5Params::readParams(Settings *settings)
 	settings->getNoiseParams("mgv5_np_cave1",        np_cave1);
 	settings->getNoiseParams("mgv5_np_cave2",        np_cave2);
 	settings->getNoiseParams("mgv5_np_ground",       np_ground);
-	settings->getNoiseParams("mgv5_np_crumble",      np_crumble);
-	settings->getNoiseParams("mgv5_np_wetness",      np_wetness);
 }
 
 
-void MapgenV5Params::writeParams(Settings *settings)
+void MapgenV5Params::writeParams(Settings *settings) const
 {
 	settings->setFlagStr("mgv5_spflags", spflags, flagdesc_mapgen_v5, (u32)-1);
 
@@ -177,8 +167,6 @@ void MapgenV5Params::writeParams(Settings *settings)
 	settings->setNoiseParams("mgv5_np_cave1",        np_cave1);
 	settings->setNoiseParams("mgv5_np_cave2",        np_cave2);
 	settings->setNoiseParams("mgv5_np_ground",       np_ground);
-	settings->setNoiseParams("mgv5_np_crumble",      np_crumble);
-	settings->setNoiseParams("mgv5_np_wetness",      np_wetness);
 }
 
 
@@ -187,23 +175,20 @@ int MapgenV5::getGroundLevelAtPoint(v2s16 p)
 	//TimeTaker t("getGroundLevelAtPoint", NULL, PRECISION_MICRO);
 
 	float f = 0.55 + NoisePerlin2D(&noise_factor->np, p.X, p.Y, seed);
-	if(f < 0.01)
+	if (f < 0.01)
 		f = 0.01;
-	else if(f >= 1.0)
+	else if (f >= 1.0)
 		f *= 1.6;
-	float h = water_level + NoisePerlin2D(&noise_height->np, p.X, p.Y, seed);
+	float h = NoisePerlin2D(&noise_height->np, p.X, p.Y, seed);
 
 	s16 search_top = water_level + 15;
 	s16 search_base = water_level;
-	// Use these 2 lines instead for a slower search returning highest ground level:
-	//s16 search_top = h + f * noise_ground->np->octaves * noise_ground->np->scale;
-	//s16 search_base = h - f * noise_ground->np->octaves * noise_ground->np->scale;
 
 	s16 level = -31000;
 	for (s16 y = search_top; y >= search_base; y--) {
 		float n_ground = NoisePerlin3D(&noise_ground->np, p.X, y, p.Y, seed);
-		if(n_ground * f > y - h) {
-			if(y >= search_top - 7)
+		if (n_ground * f > y - h) {
+			if (y >= search_top - 7)
 				break;
 			else
 				level = y;
@@ -218,6 +203,7 @@ int MapgenV5::getGroundLevelAtPoint(v2s16 p)
 
 void MapgenV5::makeChunk(BlockMakeData *data)
 {
+	// Pre-conditions
 	assert(data->vmanip);
 	assert(data->nodedef);
 	assert(data->blockpos_requested.X >= data->blockpos_min.X &&
@@ -247,33 +233,68 @@ void MapgenV5::makeChunk(BlockMakeData *data)
 
 	// Generate base terrain
 	s16 stone_surface_max_y = generateBaseTerrain();
+
+	// Create heightmap
 	updateHeightmap(node_min, node_max);
 
-	// Calculate biomes
+	// Create biomemap at heightmap surface
 	bmgr->calcBiomes(csize.X, csize.Z, noise_heat->result,
 		noise_humidity->result, heightmap, biomemap);
 
 	// Actually place the biome-specific nodes
-	generateBiomes();
+	MgStoneType stone_type = generateBiomes(noise_heat->result, noise_humidity->result);
 
 	// Generate caves
 	if ((flags & MG_CAVES) && (stone_surface_max_y >= node_min.Y))
-		generateCaves();
+		generateCaves(stone_surface_max_y);
 
 	// Generate dungeons and desert temples
 	if ((flags & MG_DUNGEONS) && (stone_surface_max_y >= node_min.Y)) {
-		DungeonGen dgen(this, NULL);
+		DungeonParams dp;
+
+		dp.np_rarity  = nparams_dungeon_rarity;
+		dp.np_density = nparams_dungeon_density;
+		dp.np_wetness = nparams_dungeon_wetness;
+		dp.c_water    = c_water_source;
+		if (stone_type == STONE) {
+			dp.c_cobble = c_cobble;
+			dp.c_moss   = c_mossycobble;
+			dp.c_stair  = c_stair_cobble;
+
+			dp.diagonal_dirs = false;
+			dp.mossratio     = 3.0;
+			dp.holesize      = v3s16(1, 2, 1);
+			dp.roomsize      = v3s16(0, 0, 0);
+			dp.notifytype    = GENNOTIFY_DUNGEON;
+		} else if (stone_type == DESERT_STONE) {
+			dp.c_cobble = c_desert_stone;
+			dp.c_moss   = c_desert_stone;
+			dp.c_stair  = c_desert_stone;
+
+			dp.diagonal_dirs = true;
+			dp.mossratio     = 0.0;
+			dp.holesize      = v3s16(2, 3, 2);
+			dp.roomsize      = v3s16(2, 5, 2);
+			dp.notifytype    = GENNOTIFY_TEMPLE;
+		} else if (stone_type == SANDSTONE) {
+			dp.c_cobble = c_sandstonebrick;
+			dp.c_moss   = c_sandstonebrick;
+			dp.c_stair  = c_sandstonebrick;
+
+			dp.diagonal_dirs = false;
+			dp.mossratio     = 0.0;
+			dp.holesize      = v3s16(2, 2, 2);
+			dp.roomsize      = v3s16(2, 0, 2);
+			dp.notifytype    = GENNOTIFY_DUNGEON;
+		}
+
+		DungeonGen dgen(this, &dp);
 		dgen.generate(blockseed, full_node_min, full_node_max);
 	}
 
 	// Generate the registered decorations
 	m_emerge->decomgr->placeAllDecos(this, blockseed, node_min, node_max);
 
-	// Generate underground dirt, sand, gravel and lava blobs
-	if (spflags & MGV5_BLOBS) {
-		generateBlobs();
-	}
-
 	// Generate the registered ores
 	m_emerge->oremgr->placeAllOres(this, blockseed, node_min, node_max);
 
@@ -311,23 +332,23 @@ void MapgenV5::calculateNoise()
 		noise_cave2->perlinMap3D(x, y, z);
 	}
 
-	if (spflags & MGV5_BLOBS) {
-		noise_crumble->perlinMap3D(x, y, z);
-		noise_wetness->perlinMap3D(x, y, z);
-	}
+	noise_filler_depth->perlinMap2D(x, z);
+	noise_heat->perlinMap2D(x, z);
+	noise_humidity->perlinMap2D(x, z);
+	noise_heat_blend->perlinMap2D(x, z);
+	noise_humidity_blend->perlinMap2D(x, z);
 
-	if (node_max.Y >= water_level) {
-		noise_filler_depth->perlinMap2D(x, z);
-		noise_heat->perlinMap2D(x, z);
-		noise_humidity->perlinMap2D(x, z);
+	for (s32 i = 0; i < csize.X * csize.Z; i++) {
+		noise_heat->result[i] += noise_heat_blend->result[i];
+		noise_humidity->result[i] += noise_humidity_blend->result[i];
 	}
 
+	heatmap = noise_heat->result;
+	humidmap = noise_humidity->result;
 	//printf("calculateNoise: %dus\n", t.stop());
 }
 
 
-// Two original MT 0.3 functions:
-
 //bool is_cave(u32 index) {
 //	double d1 = contour(noise_cave1->result[index]);
 //	double d2 = contour(noise_cave2->result[index]);
@@ -349,24 +370,24 @@ int MapgenV5::generateBaseTerrain()
 {
 	u32 index = 0;
 	u32 index2d = 0;
-	int stone_surface_max_y = -MAP_GENERATION_LIMIT;
+	int stone_surface_max_y = -MAX_MAP_GENERATION_LIMIT;
 
-	for(s16 z=node_min.Z; z<=node_max.Z; z++) {
-		for(s16 y=node_min.Y - 1; y<=node_max.Y + 1; y++) {
+	for (s16 z=node_min.Z; z<=node_max.Z; z++) {
+		for (s16 y=node_min.Y - 1; y<=node_max.Y + 1; y++) {
 			u32 i = vm->m_area.index(node_min.X, y, z);
-			for(s16 x=node_min.X; x<=node_max.X; x++, i++, index++, index2d++) {
-				if(vm->m_data[i].getContent() != CONTENT_IGNORE)
+			for (s16 x=node_min.X; x<=node_max.X; x++, i++, index++, index2d++) {
+				if (vm->m_data[i].getContent() != CONTENT_IGNORE)
 					continue;
 
 				float f = 0.55 + noise_factor->result[index2d];
-				if(f < 0.01)
+				if (f < 0.01)
 					f = 0.01;
-				else if(f >= 1.0)
+				else if (f >= 1.0)
 					f *= 1.6;
-				float h = water_level + noise_height->result[index2d];
+				float h = noise_height->result[index2d];
 
-				if(noise_ground->result[index] * f < y - h) {
-					if(y <= water_level)
+				if (noise_ground->result[index] * f < y - h) {
+					if (y <= water_level)
 						vm->m_data[i] = MapNode(c_water_source);
 					else
 						vm->m_data[i] = MapNode(CONTENT_AIR);
@@ -376,157 +397,143 @@ int MapgenV5::generateBaseTerrain()
 						stone_surface_max_y = y;
 				}
 			}
-			index2d = index2d - ystride;
+			index2d -= ystride;
 		}
-		index2d = index2d + ystride;
+		index2d += ystride;
 	}
 
 	return stone_surface_max_y;
 }
 
 
-void MapgenV5::generateBiomes()
+MgStoneType MapgenV5::generateBiomes(float *heat_map, float *humidity_map)
 {
-	if (node_max.Y < water_level)
-		return;
-
-	MapNode n_air(CONTENT_AIR);
-	MapNode n_stone(c_stone);
-	MapNode n_water(c_water_source);
-
 	v3s16 em = vm->m_area.getExtent();
 	u32 index = 0;
+	MgStoneType stone_type = STONE;
 
 	for (s16 z = node_min.Z; z <= node_max.Z; z++)
 	for (s16 x = node_min.X; x <= node_max.X; x++, index++) {
-		Biome *biome        = (Biome *)bmgr->get(biomemap[index]);
-		s16 dfiller         = biome->depth_filler + noise_filler_depth->result[index];
-		s16 y0_top          = biome->depth_top;
-		s16 y0_filler       = biome->depth_top + dfiller;
-		s16 shore_max       = water_level + biome->height_shore;
-		s16 depth_water_top = biome->depth_water_top;
+		Biome *biome = NULL;
+		u16 depth_top = 0;
+		u16 base_filler = 0;
+		u16 depth_water_top = 0;
+		u32 vi = vm->m_area.index(x, node_max.Y, z);
 
-		s16 nplaced = 0;
-		u32 i = vm->m_area.index(x, node_max.Y, z);
+		// Check node at base of mapchunk above, either a node of a previously
+		// generated mapchunk or if not, a node of overgenerated base terrain.
+		content_t c_above = vm->m_data[vi + em.X].getContent();
+		bool air_above = c_above == CONTENT_AIR;
+		bool water_above = c_above == c_water_source;
 
-		content_t c_above = vm->m_data[i + em.X].getContent();
-		bool have_air = c_above == CONTENT_AIR;
+		// If there is air or water above enable top/filler placement, otherwise force
+		// nplaced to stone level by setting a number exceeding any possible filler depth.
+		u16 nplaced = (air_above || water_above) ? 0 : (u16)-1;
 
 		for (s16 y = node_max.Y; y >= node_min.Y; y--) {
-			content_t c = vm->m_data[i].getContent();
+			content_t c = vm->m_data[vi].getContent();
 
-			if (c == c_stone && have_air) {
-				content_t c_below = vm->m_data[i - em.X].getContent();
+			// Biome is recalculated each time an upper surface is detected while
+			// working down a column. The selected biome then remains in effect for
+			// all nodes below until the next surface and biome recalculation.
+			// Biome is recalculated:
+			// 1. At the surface of stone below air or water.
+			// 2. At the surface of water below air.
+			// 3. When stone or water is detected but biome has not yet been calculated.
+			if ((c == c_stone && (air_above || water_above || !biome)) ||
+					(c == c_water_source && (air_above || !biome))) {
+				biome = bmgr->getBiome(heat_map[index], humidity_map[index], y);
+				depth_top = biome->depth_top;
+				base_filler = MYMAX(depth_top + biome->depth_filler
+						+ noise_filler_depth->result[index], 0);
+				depth_water_top = biome->depth_water_top;
 
-				if (c_below != CONTENT_AIR) {
-					if (nplaced < y0_top) {
-						if(y < water_level)
-							vm->m_data[i] = MapNode(biome->c_underwater);
-						else if(y <= shore_max)
-							vm->m_data[i] = MapNode(biome->c_shore_top);
-						else
-							vm->m_data[i] = MapNode(biome->c_top);
-						nplaced++;
-					} else if (nplaced < y0_filler && nplaced >= y0_top) {
-						if(y < water_level)
-							vm->m_data[i] = MapNode(biome->c_underwater);
-						else if(y <= shore_max)
-							vm->m_data[i] = MapNode(biome->c_shore_filler);
-						else
-							vm->m_data[i] = MapNode(biome->c_filler);
-						nplaced++;
-					} else if (c == c_stone) {
-						have_air = false;
-						nplaced  = 0;
-						vm->m_data[i] = MapNode(biome->c_stone);
-					} else {
-						have_air = false;
-						nplaced  = 0;
-					}
-				} else if (c == c_stone) {
-					have_air = false;
-					nplaced = 0;
-					vm->m_data[i] = MapNode(biome->c_stone);
-				}
-			} else if (c == c_stone) {
-				have_air = false;
-				nplaced = 0;
-				vm->m_data[i] = MapNode(biome->c_stone);
-			} else if (c == c_water_source) {
-				have_air = true;
-				nplaced = 0;
-				if(y > water_level - depth_water_top)
-					vm->m_data[i] = MapNode(biome->c_water_top);
-				else
-					vm->m_data[i] = MapNode(biome->c_water);
-			} else if (c == CONTENT_AIR) {
-				have_air = true;
-				nplaced = 0;
+				// Detect stone type for dungeons during every biome calculation.
+				// This is more efficient than detecting per-node and will not
+				// miss any desert stone or sandstone biomes.
+				if (biome->c_stone == c_desert_stone)
+					stone_type = DESERT_STONE;
+				else if (biome->c_stone == c_sandstone)
+					stone_type = SANDSTONE;
 			}
 
-			vm->m_area.add_y(em, i, -1);
+			if (c == c_stone) {
+				content_t c_below = vm->m_data[vi - em.X].getContent();
+
+				// If the node below isn't solid, make this node stone, so that
+				// any top/filler nodes above are structurally supported.
+				// This is done by aborting the cycle of top/filler placement
+				// immediately by forcing nplaced to stone level.
+				if (c_below == CONTENT_AIR || c_below == c_water_source)
+					nplaced = (u16)-1;
+
+				if (nplaced < depth_top) {
+					vm->m_data[vi] = MapNode(biome->c_top);
+					nplaced++;
+				} else if (nplaced < base_filler) {
+					vm->m_data[vi] = MapNode(biome->c_filler);
+					nplaced++;
+				} else {
+					vm->m_data[vi] = MapNode(biome->c_stone);
+				}
+
+				air_above = false;
+				water_above = false;
+			} else if (c == c_water_source) {
+				vm->m_data[vi] = MapNode((y > (s32)(water_level - depth_water_top)) ?
+						biome->c_water_top : biome->c_water);
+				nplaced = 0;  // Enable top/filler placement for next surface
+				air_above = false;
+				water_above = true;
+			} else if (c == CONTENT_AIR) {
+				nplaced = 0;  // Enable top/filler placement for next surface
+				air_above = true;
+				water_above = false;
+			} else {  // Possible various nodes overgenerated from neighbouring mapchunks
+				nplaced = (u16)-1;  // Disable top/filler placement
+				air_above = false;
+				water_above = false;
+			}
+
+			vm->m_area.add_y(em, vi, -1);
 		}
 	}
+
+	return stone_type;
 }
 
 
-void MapgenV5::generateCaves()
+void MapgenV5::generateCaves(int max_stone_y)
 {
-	u32 index = 0;
-	u32 index2d = 0;
+	if (max_stone_y >= node_min.Y) {
+		u32 index = 0;
 
-	for(s16 z=node_min.Z; z<=node_max.Z; z++) {
-		for(s16 y=node_min.Y - 1; y<=node_max.Y + 1; y++) {
+		for (s16 z = node_min.Z; z <= node_max.Z; z++)
+		for (s16 y = node_min.Y - 1; y <= node_max.Y + 1; y++) {
 			u32 i = vm->m_area.index(node_min.X, y, z);
-			for(s16 x=node_min.X; x<=node_max.X; x++, i++, index++, index2d++) {
-				Biome *biome = (Biome *)bmgr->get(biomemap[index2d]);
-				content_t c = vm->m_data[i].getContent();
-				if(c == CONTENT_AIR
-						|| (y <= water_level
-						&& c != biome->c_stone
-						&& c != c_stone))
-					continue;
-
+			for (s16 x = node_min.X; x <= node_max.X; x++, i++, index++) {
 				float d1 = contour(noise_cave1->result[index]);
 				float d2 = contour(noise_cave2->result[index]);
-				if(d1*d2 > 0.125)
+				if (d1*d2 > 0.125) {
+					content_t c = vm->m_data[i].getContent();
+					if (!ndef->get(c).is_ground_content || c == CONTENT_AIR)
+						continue;
+
 					vm->m_data[i] = MapNode(CONTENT_AIR);
-			}
-			index2d = index2d - ystride;
-		}
-		index2d = index2d + ystride;
-	}
-}
-
-
-void MapgenV5::generateBlobs()
-{
-	u32 index = 0;
-
-	for(s16 z=node_min.Z; z<=node_max.Z; z++) {
-		for(s16 y=node_min.Y - 1; y<=node_max.Y + 1; y++) {
-			u32 i = vm->m_area.index(node_min.X, y, z);
-			for(s16 x=node_min.X; x<=node_max.X; x++, i++, index++) {
-				content_t c = vm->m_data[i].getContent();
-				if(c != c_stone)
-					continue;
-
-				if(noise_crumble->result[index] > 1.3) {
-					if(noise_wetness->result[index] > 0.0)
-						vm->m_data[i] = MapNode(c_dirt);
-					else
-						vm->m_data[i] = MapNode(c_sand);
-				} else if(noise_crumble->result[index] > 0.7) {
-					if(noise_wetness->result[index] < -0.6)
-						vm->m_data[i] = MapNode(c_gravel);
-				} else if(noise_crumble->result[index] < -3.5 +
-						MYMIN(0.1 *
-						sqrt((float)MYMAX(0, -y)), 1.5)) {
-					vm->m_data[i] = MapNode(c_lava_source);
 				}
 			}
 		}
 	}
+
+	if (node_max.Y > LARGE_CAVE_DEPTH)
+		return;
+
+	PseudoRandom ps(blockseed + 21343);
+	u32 bruises_count = (ps.range(1, 4) == 1) ? ps.range(1, 2) : 0;
+	for (u32 i = 0; i < bruises_count; i++) {
+		CaveV5 cave(this, &ps);
+		cave.makeCave(node_min, node_max, max_stone_y);
+	}
 }
 
 
@@ -540,14 +547,31 @@ void MapgenV5::dustTopNodes()
 
 	for (s16 z = node_min.Z; z <= node_max.Z; z++)
 	for (s16 x = node_min.X; x <= node_max.X; x++, index++) {
-		Biome *biome = (Biome *)bmgr->get(biomemap[index]);
+		Biome *biome = (Biome *)bmgr->getRaw(biomemap[index]);
 
 		if (biome->c_dust == CONTENT_IGNORE)
 			continue;
 
-		s16 y = node_max.Y + 1;
-		u32 vi = vm->m_area.index(x, y, z);
-		for (; y >= node_min.Y; y--) {
+		u32 vi = vm->m_area.index(x, full_node_max.Y, z);
+		content_t c_full_max = vm->m_data[vi].getContent();
+		s16 y_start;
+
+		if (c_full_max == CONTENT_AIR) {
+			y_start = full_node_max.Y - 1;
+		} else if (c_full_max == CONTENT_IGNORE) {
+			vi = vm->m_area.index(x, node_max.Y + 1, z);
+			content_t c_max = vm->m_data[vi].getContent();
+
+			if (c_max == CONTENT_AIR)
+				y_start = node_max.Y;
+			else
+				continue;
+		} else {
+			continue;
+		}
+
+		vi = vm->m_area.index(x, y_start, z);
+		for (s16 y = y_start; y >= node_min.Y - 1; y--) {
 			if (vm->m_data[vi].getContent() != CONTENT_AIR)
 				break;
 
@@ -555,14 +579,9 @@ void MapgenV5::dustTopNodes()
 		}
 
 		content_t c = vm->m_data[vi].getContent();
-		if (!ndef->get(c).buildable_to && c != CONTENT_IGNORE
-				&& c != biome->c_dust) {
-			if (y == node_max.Y + 1)
-				continue;
-
+		if (!ndef->get(c).buildable_to && c != CONTENT_IGNORE && c != biome->c_dust) {
 			vm->m_area.add_y(em, vi, 1);
 			vm->m_data[vi] = MapNode(biome->c_dust);
 		}
 	}
 }
-
