@@ -21,24 +21,36 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define MAPGENV6_HEADER
 
 #include "mapgen.h"
+#include "noise.h"
 
 #define AVERAGE_MUD_AMOUNT 4
+#define DESERT_STONE_BASE -32
+#define ICE_BASE 0
+#define FREQ_HOT 0.4
+#define FREQ_SNOW -0.4
+#define FREQ_TAIGA 0.5
+#define FREQ_JUNGLE 0.5
 
-/////////////////// Mapgen V6 flags
+//////////// Mapgen V6 flags
 #define MGV6_JUNGLES    0x01
 #define MGV6_BIOMEBLEND 0x02
 #define MGV6_MUDFLOW    0x04
+#define MGV6_SNOWBIOMES 0x08
 
 
 extern FlagDesc flagdesc_mapgen_v6[];
 
 
-enum BiomeType
+enum BiomeV6Type
 {
 	BT_NORMAL,
-	BT_DESERT
+	BT_DESERT,
+	BT_JUNGLE,
+	BT_TUNDRA,
+	BT_TAIGA,
 };
 
+
 struct MapgenV6Params : public MapgenSpecificParams {
 	u32 spflags;
 	float freq_desert;
@@ -54,23 +66,22 @@ struct MapgenV6Params : public MapgenSpecificParams {
 	NoiseParams np_humidity;
 	NoiseParams np_trees;
 	NoiseParams np_apple_trees;
-	
+
 	MapgenV6Params();
 	~MapgenV6Params() {}
-	
-	void readParams(Settings *settings);
-	void writeParams(Settings *settings);
+
+	void readParams(const Settings *settings);
+	void writeParams(Settings *settings) const;
 };
 
+
 class MapgenV6 : public Mapgen {
 public:
-	EmergeManager *emerge;
+	EmergeManager *m_emerge;
 
 	int ystride;
-	u32 flags;
 	u32 spflags;
 
-	u32 blockseed;
 	v3s16 node_min;
 	v3s16 node_max;
 	v3s16 full_node_min;
@@ -85,6 +96,7 @@ public:
 	Noise *noise_mud;
 	Noise *noise_beach;
 	Noise *noise_biome;
+	Noise *noise_humidity;
 	NoiseParams *np_cave;
 	NoiseParams *np_humidity;
 	NoiseParams *np_trees;
@@ -99,23 +111,25 @@ public:
 	content_t c_water_source;
 	content_t c_lava_source;
 	content_t c_gravel;
-	content_t c_cobble;
-	content_t c_desert_sand;
 	content_t c_desert_stone;
+	content_t c_desert_sand;
+	content_t c_dirt_with_snow;
+	content_t c_snow;
+	content_t c_snowblock;
+	content_t c_ice;
 
+	content_t c_cobble;
 	content_t c_mossycobble;
-	content_t c_sandbrick;
 	content_t c_stair_cobble;
-	content_t c_stair_sandstone;
 
 	MapgenV6(int mapgenid, MapgenParams *params, EmergeManager *emerge);
 	~MapgenV6();
-	
+
 	void makeChunk(BlockMakeData *data);
 	int getGroundLevelAtPoint(v2s16 p);
 
 	float baseTerrainLevel(float terrain_base, float terrain_higher,
-						   float steepness, float height_select);
+		float steepness, float height_select);
 	virtual float baseTerrainLevelFromNoise(v2s16 p);
 	virtual float baseTerrainLevelFromMap(v2s16 p);
 	virtual float baseTerrainLevelFromMap(int index);
@@ -123,7 +137,7 @@ public:
 	s16 find_stone_level(v2s16 p2d);
 	bool block_is_underground(u64 seed, v3s16 blockpos);
 	s16 find_ground_level_from_noise(u64 seed, v2s16 p2d, s16 precision);
-	
+
 	float getHumidity(v2s16 p);
 	float getTreeAmount(v2s16 p);
 	bool getHaveAppleTree(v2s16 p);
@@ -131,30 +145,32 @@ public:
 	virtual float getMudAmount(int index);
 	bool getHaveBeach(v2s16 p);
 	bool getHaveBeach(int index);
-	BiomeType getBiome(v2s16 p);
-	BiomeType getBiome(int index, v2s16 p);
-	
+	BiomeV6Type getBiome(v2s16 p);
+	BiomeV6Type getBiome(int index, v2s16 p);
+
 	u32 get_blockseed(u64 seed, v3s16 p);
-	
+
 	virtual void calculateNoise();
 	int generateGround();
 	void addMud();
 	void flowMud(s16 &mudflow_minpos, s16 &mudflow_maxpos);
-	void addDirtGravelBlobs();
 	void growGrass();
 	void placeTreesAndJungleGrass();
 	virtual void generateCaves(int max_stone_y);
-	virtual void generateExperimental() {}
 };
 
+
 struct MapgenFactoryV6 : public MapgenFactory {
-	Mapgen *createMapgen(int mgid, MapgenParams *params, EmergeManager *emerge) {
+	Mapgen *createMapgen(int mgid, MapgenParams *params, EmergeManager *emerge)
+	{
 		return new MapgenV6(mgid, params, emerge);
 	};
-	
-	MapgenSpecificParams *createMapgenParams() {
+
+	MapgenSpecificParams *createMapgenParams()
+	{
 		return new MapgenV6Params();
 	};
 };
 
+
 #endif
