@@ -162,18 +162,13 @@ void read_object_properties(lua_State *L, int index,
 	lua_pop(L, 1);
 
 	lua_getfield(L, -1, "colors");
-	if(lua_istable(L, -1)){
-		prop->colors.clear();
+	if (lua_istable(L, -1)) {
 		int table = lua_gettop(L);
-		lua_pushnil(L);
-		while(lua_next(L, table) != 0){
-			// key at index -2 and value at index -1
-			if(lua_isstring(L, -1))
-				prop->colors.push_back(readARGB8(L, -1));
-			else
-				prop->colors.push_back(video::SColor(255, 255, 255, 255));
-			// removes value, keeps key for next iteration
-			lua_pop(L, 1);
+		prop->colors.clear();
+		for (lua_pushnil(L); lua_next(L, table); lua_pop(L, 1)) {
+			video::SColor color(255, 255, 255, 255);
+			read_color(L, -1, &color);
+			prop->colors.push_back(color);
 		}
 	}
 	lua_pop(L, 1);
@@ -205,17 +200,78 @@ void read_object_properties(lua_State *L, int index,
 }
 
 /******************************************************************************/
-TileDef read_tiledef(lua_State *L, int index)
+void push_object_properties(lua_State *L, ObjectProperties *prop)
+{
+	lua_newtable(L);
+	lua_pushnumber(L, prop->hp_max);
+	lua_setfield(L, -2, "hp_max");
+	lua_pushboolean(L, prop->physical);
+	lua_setfield(L, -2, "physical");
+	lua_pushboolean(L, prop->collideWithObjects);
+	lua_setfield(L, -2, "collide_with_objects");
+	lua_pushnumber(L, prop->weight);
+	lua_setfield(L, -2, "weight");
+	push_aabb3f(L, prop->collisionbox);
+	lua_setfield(L, -2, "collisionbox");
+	lua_pushlstring(L, prop->visual.c_str(), prop->visual.size());
+	lua_setfield(L, -2, "visual");
+	lua_pushlstring(L, prop->mesh.c_str(), prop->mesh.size());
+	lua_setfield(L, -2, "mesh");
+	push_v2f(L, prop->visual_size);
+	lua_setfield(L, -2, "visual_size");
+
+	lua_newtable(L);
+	u16 i = 1;
+	for (std::vector<std::string>::iterator it = prop->textures.begin();
+			it != prop->textures.end(); ++it) {
+		lua_pushlstring(L, it->c_str(), it->size());
+		lua_rawseti(L, -2, i);
+	}
+	lua_setfield(L, -2, "textures");
+
+	lua_newtable(L);
+	i = 1;
+	for (std::vector<video::SColor>::iterator it = prop->colors.begin();
+			it != prop->colors.end(); ++it) {
+		push_ARGB8(L, *it);
+		lua_rawseti(L, -2, i);
+	}
+	lua_setfield(L, -2, "colors");
+
+	push_v2s16(L, prop->spritediv);
+	lua_setfield(L, -2, "spritediv");
+	push_v2s16(L, prop->initial_sprite_basepos);
+	lua_setfield(L, -2, "initial_sprite_basepos");
+	lua_pushboolean(L, prop->is_visible);
+	lua_setfield(L, -2, "is_visible");
+	lua_pushboolean(L, prop->makes_footstep_sound);
+	lua_setfield(L, -2, "makes_footstep_sound");
+	lua_pushnumber(L, prop->automatic_rotate);
+	lua_setfield(L, -2, "automatic_rotate");
+	lua_pushnumber(L, prop->stepheight / BS);
+	lua_setfield(L, -2, "stepheight");
+	if (prop->automatic_face_movement_dir)
+		lua_pushnumber(L, prop->automatic_face_movement_dir_offset);
+	else
+		lua_pushboolean(L, false);
+	lua_setfield(L, -2, "automatic_face_movement_dir");
+}
+
+/******************************************************************************/
+TileDef read_tiledef(lua_State *L, int index, u8 drawtype)
 {
 	if(index < 0)
 		index = lua_gettop(L) + 1 + index;
 
 	TileDef tiledef;
-
+	bool default_tiling = (drawtype == NDT_PLANTLIKE || drawtype == NDT_FIRELIKE)
+		? false : true;
 	// key at index -2 and value at index
 	if(lua_isstring(L, index)){
 		// "default_lava.png"
 		tiledef.name = lua_tostring(L, index);
+		tiledef.tileable_vertical = default_tiling;
+		tiledef.tileable_horizontal = default_tiling;
 	}
 	else if(lua_istable(L, index))
 	{
@@ -224,20 +280,24 @@ TileDef read_tiledef(lua_State *L, int index)
 		getstringfield(L, index, "name", tiledef.name);
 		getstringfield(L, index, "image", tiledef.name); // MaterialSpec compat.
 		tiledef.backface_culling = getboolfield_default(
-					L, index, "backface_culling", true);
+			L, index, "backface_culling", true);
+		tiledef.tileable_horizontal = getboolfield_default(
+			L, index, "tileable_horizontal", default_tiling);
+		tiledef.tileable_vertical = getboolfield_default(
+			L, index, "tileable_vertical", default_tiling);
 		// animation = {}
 		lua_getfield(L, index, "animation");
 		if(lua_istable(L, -1)){
 			// {type="vertical_frames", aspect_w=16, aspect_h=16, length=2.0}
 			tiledef.animation.type = (TileAnimationType)
-					getenumfield(L, -1, "type", es_TileAnimationType,
-					TAT_NONE);
+				getenumfield(L, -1, "type", es_TileAnimationType,
+				TAT_NONE);
 			tiledef.animation.aspect_w =
-					getintfield_default(L, -1, "aspect_w", 16);
+				getintfield_default(L, -1, "aspect_w", 16);
 			tiledef.animation.aspect_h =
-					getintfield_default(L, -1, "aspect_h", 16);
+				getintfield_default(L, -1, "aspect_h", 16);
 			tiledef.animation.length =
-					getfloatfield_default(L, -1, "length", 1.0);
+				getfloatfield_default(L, -1, "length", 1.0);
 		}
 		lua_pop(L, 1);
 	}
@@ -300,7 +360,7 @@ ContentFeatures read_content_features(lua_State *L, int index)
 		int i = 0;
 		while(lua_next(L, table) != 0){
 			// Read tiledef from value
-			f.tiledef[i] = read_tiledef(L, -1);
+			f.tiledef[i] = read_tiledef(L, -1, f.drawtype);
 			// removes value, keeps key for next iteration
 			lua_pop(L, 1);
 			i++;
@@ -335,7 +395,7 @@ ContentFeatures read_content_features(lua_State *L, int index)
 		int i = 0;
 		while(lua_next(L, table) != 0){
 			// Read tiledef from value
-			f.tiledef_special[i] = read_tiledef(L, -1);
+			f.tiledef_special[i] = read_tiledef(L, -1, f.drawtype);
 			// removes value, keeps key for next iteration
 			lua_pop(L, 1);
 			i++;
@@ -357,8 +417,7 @@ ContentFeatures read_content_features(lua_State *L, int index)
 	/* Other stuff */
 
 	lua_getfield(L, index, "post_effect_color");
-	if(!lua_isnil(L, -1))
-		f.post_effect_color = readARGB8(L, -1);
+	read_color(L, -1, &f.post_effect_color);
 	lua_pop(L, 1);
 
 	f.param_type = (ContentParamType)getenumfield(L, index, "paramtype",
@@ -414,7 +473,6 @@ ContentFeatures read_content_features(lua_State *L, int index)
 	f.leveled = getintfield_default(L, index, "leveled", f.leveled);
 
 	getboolfield(L, index, "liquid_renewable", f.liquid_renewable);
-	getstringfield(L, index, "freezemelt", f.freezemelt);
 	f.drowning = getintfield_default(L, index,
 			"drowning", f.drowning);
 	// Amount of light the node emits
@@ -547,22 +605,23 @@ NodeBox read_nodebox(lua_State *L, int index)
 MapNode readnode(lua_State *L, int index, INodeDefManager *ndef)
 {
 	lua_getfield(L, index, "name");
-	const char *name = luaL_checkstring(L, -1);
+	if (!lua_isstring(L, -1))
+		throw LuaError("Node name is not set or is not a string!");
+	const char *name = lua_tostring(L, -1);
 	lua_pop(L, 1);
-	u8 param1;
+
+	u8 param1 = 0;
 	lua_getfield(L, index, "param1");
-	if(lua_isnil(L, -1))
-		param1 = 0;
-	else
+	if (!lua_isnil(L, -1))
 		param1 = lua_tonumber(L, -1);
 	lua_pop(L, 1);
-	u8 param2;
+
+	u8 param2 = 0;
 	lua_getfield(L, index, "param2");
-	if(lua_isnil(L, -1))
-		param2 = 0;
-	else
+	if (!lua_isnil(L, -1))
 		param2 = lua_tonumber(L, -1);
 	lua_pop(L, 1);
+
 	return MapNode(ndef, name, param1, param2);
 }
 
@@ -902,6 +961,12 @@ u32 read_flags_table(lua_State *L, int table, FlagDesc *flagdesc, u32 *flagmask)
 	return flags;
 }
 
+void push_flags_string(lua_State *L, FlagDesc *flagdesc, u32 flags, u32 flagmask)
+{
+	std::string flagstring = writeFlagString(flags, flagdesc, flagmask);
+	lua_pushlstring(L, flagstring.c_str(), flagstring.size());
+}
+
 /******************************************************************************/
 /* Lua Stored data!                                                           */
 /******************************************************************************/
@@ -926,15 +991,24 @@ void read_groups(lua_State *L, int index,
 	}
 }
 
+/******************************************************************************/
+void push_groups(lua_State *L, const std::map<std::string, int> &groups)
+{
+	lua_newtable(L);
+	std::map<std::string, int>::const_iterator it;
+	for (it = groups.begin(); it != groups.end(); ++it) {
+		lua_pushnumber(L, it->second);
+		lua_setfield(L, -2, it->first.c_str());
+	}
+}
+
 /******************************************************************************/
 void push_items(lua_State *L, const std::vector<ItemStack> &items)
 {
-	// Create and fill table
 	lua_createtable(L, items.size(), 0);
-	std::vector<ItemStack>::const_iterator iter = items.begin();
-	for (u32 i = 0; iter != items.end(); iter++) {
-		LuaItemStack::create(L, *iter);
-		lua_rawseti(L, -2, ++i);
+	for (u32 i = 0; i != items.size(); i++) {
+		LuaItemStack::create(L, items[i]);
+		lua_rawseti(L, -2, i + 1);
 	}
 }
 
@@ -983,14 +1057,16 @@ bool read_noiseparams(lua_State *L, int index, NoiseParams *np)
 	if (!lua_istable(L, index))
 		return false;
 
-	np->offset     = getfloatfield_default(L, index, "offset",     0.0);
-	np->scale      = getfloatfield_default(L, index, "scale",      0.0);
-	np->persist    = getfloatfield_default(L, index, "persist",    0.0);
-	np->lacunarity = getfloatfield_default(L, index, "lacunarity", 2.0);
-	np->seed       = getintfield_default(L,   index, "seed",       0);
-	np->octaves    = getintfield_default(L,   index, "octaves",    0);
+	getfloatfield(L, index, "offset",      np->offset);
+	getfloatfield(L, index, "scale",       np->scale);
+	getfloatfield(L, index, "persist",     np->persist);
+	getfloatfield(L, index, "persistence", np->persist);
+	getfloatfield(L, index, "lacunarity",  np->lacunarity);
+	getintfield(L,   index, "seed",        np->seed);
+	getintfield(L,   index, "octaves",     np->octaves);
 
-	u32 flags = 0, flagmask = 0;
+	u32 flags    = 0;
+	u32 flagmask = 0;
 	np->flags = getflagsfield(L, index, "flags", flagdesc_noiseparams,
 		&flags, &flagmask) ? flags : NOISE_FLAG_DEFAULTS;
 
@@ -1001,6 +1077,30 @@ bool read_noiseparams(lua_State *L, int index, NoiseParams *np)
 	return true;
 }
 
+void push_noiseparams(lua_State *L, NoiseParams *np)
+{
+	lua_newtable(L);
+	lua_pushnumber(L, np->offset);
+	lua_setfield(L, -2, "offset");
+	lua_pushnumber(L, np->scale);
+	lua_setfield(L, -2, "scale");
+	lua_pushnumber(L, np->persist);
+	lua_setfield(L, -2, "persistence");
+	lua_pushnumber(L, np->lacunarity);
+	lua_setfield(L, -2, "lacunarity");
+	lua_pushnumber(L, np->seed);
+	lua_setfield(L, -2, "seed");
+	lua_pushnumber(L, np->octaves);
+	lua_setfield(L, -2, "octaves");
+
+	push_flags_string(L, flagdesc_noiseparams, np->flags,
+		np->flags);
+	lua_setfield(L, -2, "flags");
+
+	push_v3f(L, np->spread);
+	lua_setfield(L, -2, "spread");
+}
+
 /******************************************************************************/
 // Returns depth of json value tree
 static int push_json_value_getdepth(const Json::Value &value)
