@@ -31,15 +31,15 @@ bool ScriptApiEntity::luaentity_Add(u16 id, const char *name)
 	verbosestream<<"scriptapi_luaentity_add: id="<<id<<" name=\""
 			<<name<<"\""<<std::endl;
 
-	// Get minetest.registered_entities[name]
-	lua_getglobal(L, "minetest");
+	// Get core.registered_entities[name]
+	lua_getglobal(L, "core");
 	lua_getfield(L, -1, "registered_entities");
 	luaL_checktype(L, -1, LUA_TTABLE);
 	lua_pushstring(L, name);
 	lua_gettable(L, -2);
 	// Should be a table, which we will use as a prototype
 	//luaL_checktype(L, -1, LUA_TTABLE);
-	if(lua_type(L, -1) != LUA_TTABLE){
+	if (lua_type(L, -1) != LUA_TTABLE){
 		errorstream<<"LuaEntity name \""<<name<<"\" not defined"<<std::endl;
 		return false;
 	}
@@ -56,14 +56,14 @@ bool ScriptApiEntity::luaentity_Add(u16 id, const char *name)
 
 	// Add object reference
 	// This should be userdata with metatable ObjectRef
-	objectrefGet(id);
+	objectrefGet(L, id);
 	luaL_checktype(L, -1, LUA_TUSERDATA);
-	if(!luaL_checkudata(L, -1, "ObjectRef"))
+	if (!luaL_checkudata(L, -1, "ObjectRef"))
 		luaL_typerror(L, -1, "ObjectRef");
 	lua_setfield(L, -2, "object");
 
-	// minetest.luaentities[id] = object
-	lua_getglobal(L, "minetest");
+	// core.luaentities[id] = object
+	lua_getglobal(L, "core");
 	lua_getfield(L, -1, "luaentities");
 	luaL_checktype(L, -1, LUA_TTABLE);
 	lua_pushnumber(L, id); // Push id
@@ -78,39 +78,36 @@ void ScriptApiEntity::luaentity_Activate(u16 id,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
+	verbosestream << "scriptapi_luaentity_activate: id=" << id << std::endl;
 
-	verbosestream<<"scriptapi_luaentity_activate: id="<<id<<std::endl;
-
-	// Get minetest.luaentities[id]
+	// Get core.luaentities[id]
 	luaentity_get(L, id);
 	int object = lua_gettop(L);
 
 	// Get on_activate function
 	lua_getfield(L, -1, "on_activate");
-	if(!lua_isnil(L, -1)) {
+	if (!lua_isnil(L, -1)) {
 		luaL_checktype(L, -1, LUA_TFUNCTION);
 		lua_pushvalue(L, object); // self
 		lua_pushlstring(L, staticdata.c_str(), staticdata.size());
 		lua_pushinteger(L, dtime_s);
-		// Call with 3 arguments, 0 results
-		if(lua_pcall(L, 3, 0, errorhandler))
-			scriptError();
+
+		setOriginFromTable(object);
+		PCALL_RES(lua_pcall(L, 3, 0, m_errorhandler));
 	} else {
 		lua_pop(L, 1);
 	}
-	lua_pop(L, 2); // Pop object and error handler
+	lua_pop(L, 1); // Pop object
 }
 
 void ScriptApiEntity::luaentity_Remove(u16 id)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	verbosestream<<"scriptapi_luaentity_rm: id="<<id<<std::endl;
+	verbosestream << "scriptapi_luaentity_rm: id=" << id << std::endl;
 
-	// Get minetest.luaentities table
-	lua_getglobal(L, "minetest");
+	// Get core.luaentities table
+	lua_getglobal(L, "core");
 	lua_getfield(L, -1, "luaentities");
 	luaL_checktype(L, -1, LUA_TTABLE);
 	int objectstable = lua_gettop(L);
@@ -120,37 +117,36 @@ void ScriptApiEntity::luaentity_Remove(u16 id)
 	lua_pushnil(L);
 	lua_settable(L, objectstable);
 
-	lua_pop(L, 2); // pop luaentities, minetest
+	lua_pop(L, 2); // pop luaentities, core
 }
 
 std::string ScriptApiEntity::luaentity_GetStaticdata(u16 id)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	//infostream<<"scriptapi_luaentity_get_staticdata: id="<<id<<std::endl;
 
-	// Get minetest.luaentities[id]
+	// Get core.luaentities[id]
 	luaentity_get(L, id);
 	int object = lua_gettop(L);
 
 	// Get get_staticdata function
 	lua_getfield(L, -1, "get_staticdata");
-	if(lua_isnil(L, -1))
+	if (lua_isnil(L, -1)) {
+		lua_pop(L, 2); // Pop entity and  get_staticdata
 		return "";
-
+	}
 	luaL_checktype(L, -1, LUA_TFUNCTION);
 	lua_pushvalue(L, object); // self
-	// Call with 1 arguments, 1 results
-	if(lua_pcall(L, 1, 1, errorhandler))
-		scriptError();
+
+	setOriginFromTable(object);
+	PCALL_RES(lua_pcall(L, 1, 1, m_errorhandler));
+
 	lua_remove(L, object); // Remove object
-	lua_remove(L, errorhandler); // Remove error handler
 
 	size_t len = 0;
 	const char *s = lua_tolstring(L, -1, &len);
+	lua_pop(L, 1); // Pop static data
 	return std::string(s, len);
 }
 
@@ -161,9 +157,8 @@ void ScriptApiEntity::luaentity_GetProperties(u16 id,
 
 	//infostream<<"scriptapi_luaentity_get_properties: id="<<id<<std::endl;
 
-	// Get minetest.luaentities[id]
-	luaentity_get(L,id);
-	//int object = lua_gettop(L);
+	// Get core.luaentities[id]
+	luaentity_get(L, id);
 
 	// Set default values that differ from ObjectProperties defaults
 	prop->hp_max = 10;
@@ -178,7 +173,7 @@ void ScriptApiEntity::luaentity_GetProperties(u16 id,
 	getfloatfield(L, -1, "weight", prop->weight);
 
 	lua_getfield(L, -1, "collisionbox");
-	if(lua_istable(L, -1))
+	if (lua_istable(L, -1))
 		prop->collisionbox = read_aabb3f(L, -1, 1.0);
 	lua_pop(L, 1);
 
@@ -199,27 +194,26 @@ void ScriptApiEntity::luaentity_Step(u16 id, float dtime)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	//infostream<<"scriptapi_luaentity_step: id="<<id<<std::endl;
 
-	// Get minetest.luaentities[id]
+	// Get core.luaentities[id]
 	luaentity_get(L, id);
 	int object = lua_gettop(L);
 	// State: object is at top of stack
 	// Get step function
 	lua_getfield(L, -1, "on_step");
-	if(lua_isnil(L, -1))
+	if (lua_isnil(L, -1)) {
+		lua_pop(L, 2); // Pop on_step and entity
 		return;
+	}
 	luaL_checktype(L, -1, LUA_TFUNCTION);
 	lua_pushvalue(L, object); // self
 	lua_pushnumber(L, dtime); // dtime
-	// Call with 2 arguments, 0 results
-	if(lua_pcall(L, 2, 0, errorhandler))
-		scriptError();
-	lua_remove(L, object); // Remove object
-	lua_remove(L, errorhandler); // Remove error handler
+
+	setOriginFromTable(object);
+	PCALL_RES(lua_pcall(L, 2, 0, m_errorhandler));
+
+	lua_pop(L, 1); // Pop object
 }
 
 // Calls entity:on_punch(ObjectRef puncher, time_from_last_punch,
@@ -230,30 +224,29 @@ void ScriptApiEntity::luaentity_Punch(u16 id,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	//infostream<<"scriptapi_luaentity_step: id="<<id<<std::endl;
 
-	// Get minetest.luaentities[id]
+	// Get core.luaentities[id]
 	luaentity_get(L,id);
 	int object = lua_gettop(L);
 	// State: object is at top of stack
 	// Get function
 	lua_getfield(L, -1, "on_punch");
-	if(lua_isnil(L, -1))
+	if (lua_isnil(L, -1)) {
+		lua_pop(L, 2); // Pop on_punch and entitu
 		return;
+	}
 	luaL_checktype(L, -1, LUA_TFUNCTION);
-	lua_pushvalue(L, object); // self
-	objectrefGetOrCreate(puncher); // Clicker reference
+	lua_pushvalue(L, object);  // self
+	objectrefGetOrCreate(L, puncher);  // Clicker reference
 	lua_pushnumber(L, time_from_last_punch);
 	push_tool_capabilities(L, *toolcap);
 	push_v3f(L, dir);
-	// Call with 5 arguments, 0 results
-	if(lua_pcall(L, 5, 0, errorhandler))
-		scriptError();
-	lua_remove(L, object); // Remove object
-	lua_remove(L, errorhandler); // Remove error handler
+
+	setOriginFromTable(object);
+	PCALL_RES(lua_pcall(L, 5, 0, m_errorhandler));
+
+	lua_pop(L, 1); // Pop object
 }
 
 // Calls entity:on_rightclick(ObjectRef clicker)
@@ -262,26 +255,25 @@ void ScriptApiEntity::luaentity_Rightclick(u16 id,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	//infostream<<"scriptapi_luaentity_step: id="<<id<<std::endl;
 
-	// Get minetest.luaentities[id]
-	luaentity_get(L,id);
+	// Get core.luaentities[id]
+	luaentity_get(L, id);
 	int object = lua_gettop(L);
 	// State: object is at top of stack
 	// Get function
 	lua_getfield(L, -1, "on_rightclick");
-	if(lua_isnil(L, -1))
+	if (lua_isnil(L, -1)) {
+		lua_pop(L, 2); // Pop on_rightclick and entity
 		return;
+	}
 	luaL_checktype(L, -1, LUA_TFUNCTION);
 	lua_pushvalue(L, object); // self
-	objectrefGetOrCreate(clicker); // Clicker reference
-	// Call with 2 arguments, 0 results
-	if(lua_pcall(L, 2, 0, errorhandler))
-		scriptError();
-	lua_remove(L, object); // Remove object
-	lua_remove(L, errorhandler); // Remove error handler
+	objectrefGetOrCreate(L, clicker); // Clicker reference
+
+	setOriginFromTable(object);
+	PCALL_RES(lua_pcall(L, 2, 0, m_errorhandler));
+
+	lua_pop(L, 1); // Pop object
 }
 
