@@ -21,17 +21,16 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define S_MAINMENU_H_
 
 #include "cpp_api/s_base.h"
-#include <map>
+#include "util/string.h"
+#include "../guiMainMenu.h"
 
-class ScriptApiMainMenu
-		: virtual public ScriptApiBase
-{
+class ScriptApiMainMenu : virtual public ScriptApiBase {
 public:
 	/**
-	 * set gamedata.errormessage to inform lua of an error
-	 * @param errormessage the error message
+	 * Hand over MainMenuDataForScript to lua to inform lua of the content
+	 * @param data the data
 	 */
-	void setMainMenuErrorMessage(std::string errormessage);
+	void setMainMenuData(MainMenuDataForScript *data);
 
 	/**
 	 * process events received from formspec
@@ -43,7 +42,7 @@ public:
 	 * process field data recieved from formspec
 	 * @param fields data in field format
 	 */
-	void handleMainMenuButtons(std::map<std::string, std::string> fields);
+	void handleMainMenuButtons(const StringMap &fields);
 };
 
 #endif /* S_MAINMENU_H_ */
