@@ -24,6 +24,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "nodedef.h"
 #include "server.h"
 #include "environment.h"
+#include "util/pointedthing.h"
 
 
 struct EnumString ScriptApiNode::es_DrawType[] =
@@ -34,14 +35,17 @@ struct EnumString ScriptApiNode::es_DrawType[] =
 		{NDT_FLOWINGLIQUID, "flowingliquid"},
 		{NDT_GLASSLIKE, "glasslike"},
 		{NDT_GLASSLIKE_FRAMED, "glasslike_framed"},
+		{NDT_GLASSLIKE_FRAMED_OPTIONAL, "glasslike_framed_optional"},
 		{NDT_ALLFACES, "allfaces"},
 		{NDT_ALLFACES_OPTIONAL, "allfaces_optional"},
 		{NDT_TORCHLIKE, "torchlike"},
 		{NDT_SIGNLIKE, "signlike"},
 		{NDT_PLANTLIKE, "plantlike"},
+		{NDT_FIRELIKE, "firelike"},
 		{NDT_FENCELIKE, "fencelike"},
 		{NDT_RAILLIKE, "raillike"},
 		{NDT_NODEBOX, "nodebox"},
+		{NDT_MESH, "mesh"},
 		{0, NULL},
 	};
 
@@ -87,26 +91,22 @@ ScriptApiNode::~ScriptApiNode() {
 }
 
 bool ScriptApiNode::node_on_punch(v3s16 p, MapNode node,
-		ServerActiveObject *puncher)
+		ServerActiveObject *puncher, PointedThing pointed)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	INodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
-	if(!getItemCallback(ndef->get(node).name.c_str(), "on_punch"))
+	if (!getItemCallback(ndef->get(node).name.c_str(), "on_punch"))
 		return false;
 
 	// Call function
 	push_v3s16(L, p);
 	pushnode(L, node, ndef);
-	objectrefGetOrCreate(puncher);
-	if(lua_pcall(L, 3, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	objectrefGetOrCreate(L, puncher);
+	pushPointedThing(pointed);
+	PCALL_RES(lua_pcall(L, 4, 0, m_errorhandler));
 	return true;
 }
 
@@ -115,22 +115,17 @@ bool ScriptApiNode::node_on_dig(v3s16 p, MapNode node,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	INodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
-	if(!getItemCallback(ndef->get(node).name.c_str(), "on_dig"))
+	if (!getItemCallback(ndef->get(node).name.c_str(), "on_dig"))
 		return false;
 
 	// Call function
 	push_v3s16(L, p);
 	pushnode(L, node, ndef);
-	objectrefGetOrCreate(digger);
-	if(lua_pcall(L, 3, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	objectrefGetOrCreate(L, digger);
+	PCALL_RES(lua_pcall(L, 3, 0, m_errorhandler));
 	return true;
 }
 
@@ -138,149 +133,113 @@ void ScriptApiNode::node_on_construct(v3s16 p, MapNode node)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	INodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
-	if(!getItemCallback(ndef->get(node).name.c_str(), "on_construct"))
+	if (!getItemCallback(ndef->get(node).name.c_str(), "on_construct"))
 		return;
 
 	// Call function
 	push_v3s16(L, p);
-	if(lua_pcall(L, 1, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	PCALL_RES(lua_pcall(L, 1, 0, m_errorhandler));
 }
 
 void ScriptApiNode::node_on_destruct(v3s16 p, MapNode node)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	INodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
-	if(!getItemCallback(ndef->get(node).name.c_str(), "on_destruct"))
+	if (!getItemCallback(ndef->get(node).name.c_str(), "on_destruct"))
 		return;
 
 	// Call function
 	push_v3s16(L, p);
-	if(lua_pcall(L, 1, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	PCALL_RES(lua_pcall(L, 1, 0, m_errorhandler));
 }
 
 void ScriptApiNode::node_after_destruct(v3s16 p, MapNode node)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	INodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
-	if(!getItemCallback(ndef->get(node).name.c_str(), "after_destruct"))
+	if (!getItemCallback(ndef->get(node).name.c_str(), "after_destruct"))
 		return;
 
 	// Call function
 	push_v3s16(L, p);
 	pushnode(L, node, ndef);
-	if(lua_pcall(L, 2, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	PCALL_RES(lua_pcall(L, 2, 0, m_errorhandler));
 }
 
 bool ScriptApiNode::node_on_timer(v3s16 p, MapNode node, f32 dtime)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	INodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
-	if(!getItemCallback(ndef->get(node).name.c_str(), "on_timer"))
+	if (!getItemCallback(ndef->get(node).name.c_str(), "on_timer"))
 		return false;
 
 	// Call function
 	push_v3s16(L, p);
 	lua_pushnumber(L,dtime);
-	if(lua_pcall(L, 2, 1, errorhandler))
-		scriptError();
-	lua_remove(L, errorhandler); // Remove error handler
+	PCALL_RES(lua_pcall(L, 2, 1, m_errorhandler));
 	return (bool) lua_isboolean(L, -1) && (bool) lua_toboolean(L, -1) == true;
 }
 
 void ScriptApiNode::node_on_receive_fields(v3s16 p,
 		const std::string &formname,
-		const std::map<std::string, std::string> &fields,
+		const StringMap &fields,
 		ServerActiveObject *sender)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	INodeDefManager *ndef = getServer()->ndef();
 
 	// If node doesn't exist, we don't know what callback to call
 	MapNode node = getEnv()->getMap().getNodeNoEx(p);
-	if(node.getContent() == CONTENT_IGNORE)
+	if (node.getContent() == CONTENT_IGNORE)
 		return;
 
 	// Push callback function on stack
-	if(!getItemCallback(ndef->get(node).name.c_str(), "on_receive_fields"))
+	if (!getItemCallback(ndef->get(node).name.c_str(), "on_receive_fields"))
 		return;
 
 	// Call function
 	push_v3s16(L, p);                    // pos
 	lua_pushstring(L, formname.c_str()); // formname
 	lua_newtable(L);                     // fields
-	for(std::map<std::string, std::string>::const_iterator
-			i = fields.begin(); i != fields.end(); i++){
-		const std::string &name = i->first;
-		const std::string &value = i->second;
+	StringMap::const_iterator it;
+	for (it = fields.begin(); it != fields.end(); it++) {
+		const std::string &name = it->first;
+		const std::string &value = it->second;
 		lua_pushstring(L, name.c_str());
 		lua_pushlstring(L, value.c_str(), value.size());
 		lua_settable(L, -3);
 	}
-	objectrefGetOrCreate(sender);        // player
-	if(lua_pcall(L, 4, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	objectrefGetOrCreate(L, sender);        // player
+	PCALL_RES(lua_pcall(L, 4, 0, m_errorhandler));
 }
 
 void ScriptApiNode::node_falling_update(v3s16 p)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	lua_getglobal(L, "nodeupdate");
 	push_v3s16(L, p);
-	if(lua_pcall(L, 1, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	PCALL_RES(lua_pcall(L, 1, 0, m_errorhandler));
 }
 
 void ScriptApiNode::node_falling_update_single(v3s16 p)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	lua_getglobal(L, "nodeupdate_single");
 	push_v3s16(L, p);
-	if(lua_pcall(L, 1, 0, errorhandler))
-		scriptError();
-	lua_pop(L, 1); // Pop error handler
+	PCALL_RES(lua_pcall(L, 1, 0, m_errorhandler));
 }
-
