@@ -49,7 +49,7 @@ void LuaABM::trigger(ServerEnvironment *env, v3s16 p, MapNode n,
 	scriptIface->realityCheck();
 
 	lua_State *L = scriptIface->getStack();
-	assert(lua_checkstack(L, 20));
+	sanity_check(lua_checkstack(L, 20));
 	StackUnroller stack_unroller(L);
 
 	lua_pushcfunction(L, script_error_handler);
@@ -65,9 +65,11 @@ void LuaABM::trigger(ServerEnvironment *env, v3s16 p, MapNode n,
 	lua_pushnumber(L, m_id);
 	lua_gettable(L, -2);
 	if(lua_isnil(L, -1))
-		assert(0);
+		FATAL_ERROR("");
 	lua_remove(L, -2); // Remove registered_abms
 
+	scriptIface->setOriginFromTable(-1);
+
 	// Call action
 	luaL_checktype(L, -1, LUA_TTABLE);
 	lua_getfield(L, -1, "action");
@@ -77,8 +79,11 @@ void LuaABM::trigger(ServerEnvironment *env, v3s16 p, MapNode n,
 	pushnode(L, n, env->getGameDef()->ndef());
 	lua_pushnumber(L, active_object_count);
 	lua_pushnumber(L, active_object_count_wider);
-	if(lua_pcall(L, 4, 0, errorhandler))
-		script_error(L);
+
+	int result = lua_pcall(L, 4, 0, errorhandler);
+	if (result)
+		scriptIface->scriptError(result, "LuaABM::trigger");
+
 	lua_pop(L, 1); // Pop error handler
 }
 
@@ -334,6 +339,22 @@ int ModApiEnvMod::l_add_node_level(lua_State *L)
 	return 1;
 }
 
+// find_nodes_with_meta(pos1, pos2)
+int ModApiEnvMod::l_find_nodes_with_meta(lua_State *L)
+{
+	GET_ENV_PTR;
+
+	std::vector<v3s16> positions = env->getMap().findNodesWithMetadata(
+		check_v3s16(L, 1), check_v3s16(L, 2));
+
+	lua_newtable(L);
+	for (size_t i = 0; i != positions.size(); i++) {
+		push_v3s16(L, positions[i]);
+		lua_rawseti(L, -2, i + 1);
+	}
+
+	return 1;
+}
 
 // get_meta(pos)
 int ModApiEnvMod::l_get_meta(lua_State *L)
@@ -402,23 +423,11 @@ int ModApiEnvMod::l_add_item(lua_State *L)
 		return 0;
 	lua_pushvalue(L, 1);
 	lua_pushstring(L, item.getItemString().c_str());
-	if(lua_pcall(L, 2, 1, errorhandler))
-		script_error(L);
+
+	PCALL_RESL(L, lua_pcall(L, 2, 1, errorhandler));
+
 	lua_remove(L, errorhandler); // Remove error handler
 	return 1;
-	/*lua_pushvalue(L, 1);
-	lua_pushstring(L, "__builtin:item");
-	lua_pushstring(L, item.getItemString().c_str());
-	return l_add_entity(L);*/
-	/*// Do it
-	ServerActiveObject *obj = createItemSAO(env, pos, item.getItemString());
-	int objectid = env->addActiveObject(obj);
-	// If failed to add, return nothing (reads as nil)
-	if(objectid == 0)
-		return 0;
-	// Return ObjectRef
-	objectrefGetOrCreate(L, obj);
-	return 1;*/
 }
 
 // get_player_by_name(name)
@@ -451,10 +460,11 @@ int ModApiEnvMod::l_get_objects_inside_radius(lua_State *L)
 	// Do it
 	v3f pos = checkFloatPos(L, 1);
 	float radius = luaL_checknumber(L, 2) * BS;
-	std::set<u16> ids = env->getObjectsInsideRadius(pos, radius);
+	std::vector<u16> ids;
+	env->getObjectsInsideRadius(ids, pos, radius);
 	ScriptApiBase *script = getScriptApiBase(L);
 	lua_createtable(L, ids.size(), 0);
-	std::set<u16>::const_iterator iter = ids.begin();
+	std::vector<u16>::const_iterator iter = ids.begin();
 	for(u32 i = 0; iter != ids.end(); iter++) {
 		ServerActiveObject *obj = env->getActiveObject(*iter);
 		// Insert object reference into table
@@ -472,7 +482,7 @@ int ModApiEnvMod::l_set_timeofday(lua_State *L)
 
 	// Do it
 	float timeofday_f = luaL_checknumber(L, 1);
-	assert(timeofday_f >= 0.0 && timeofday_f <= 1.0);
+	sanity_check(timeofday_f >= 0.0 && timeofday_f <= 1.0);
 	int timeofday_mh = (int)(timeofday_f * 24000.0);
 	// This should be set directly in the environment but currently
 	// such changes aren't immediately sent to the clients, so call
@@ -530,9 +540,8 @@ int ModApiEnvMod::l_find_node_near(lua_State *L)
 	}
 
 	for(int d=1; d<=radius; d++){
-		std::list<v3s16> list;
-		getFacePositions(list, d);
-		for(std::list<v3s16>::iterator i = list.begin();
+		std::vector<v3s16> list = FacePositionCache::getFacePositions(d);
+		for(std::vector<v3s16>::iterator i = list.begin();
 				i != list.end(); ++i){
 			v3s16 p = pos + (*i);
 			content_t c = env->getMap().getNodeNoEx(p).getContent();
@@ -555,30 +564,92 @@ int ModApiEnvMod::l_find_nodes_in_area(lua_State *L)
 	v3s16 minp = read_v3s16(L, 1);
 	v3s16 maxp = read_v3s16(L, 2);
 	std::set<content_t> filter;
-	if(lua_istable(L, 3)){
+	if(lua_istable(L, 3)) {
 		int table = 3;
 		lua_pushnil(L);
-		while(lua_next(L, table) != 0){
+		while(lua_next(L, table) != 0) {
 			// key at index -2 and value at index -1
 			luaL_checktype(L, -1, LUA_TSTRING);
 			ndef->getIds(lua_tostring(L, -1), filter);
 			// removes value, keeps key for next iteration
 			lua_pop(L, 1);
 		}
-	} else if(lua_isstring(L, 3)){
+	} else if(lua_isstring(L, 3)) {
+		ndef->getIds(lua_tostring(L, 3), filter);
+	}
+
+	std::map<content_t, u16> individual_count;
+
+	lua_newtable(L);
+	u64 i = 0;
+	for (s16 x = minp.X; x <= maxp.X; x++)
+		for (s16 y = minp.Y; y <= maxp.Y; y++)
+			for (s16 z = minp.Z; z <= maxp.Z; z++) {
+				v3s16 p(x, y, z);
+				content_t c = env->getMap().getNodeNoEx(p).getContent();
+				if (filter.count(c) != 0) {
+					push_v3s16(L, p);
+					lua_rawseti(L, -2, ++i);
+					individual_count[c]++;
+				}
+	}
+	lua_newtable(L);
+	for (std::set<content_t>::iterator it = filter.begin();
+			it != filter.end(); ++it) {
+		lua_pushnumber(L, individual_count[*it]);
+		lua_setfield(L, -2, ndef->get(*it).name.c_str());
+	}
+	return 2;
+}
+
+// find_nodes_in_area_under_air(minp, maxp, nodenames) -> list of positions
+// nodenames: e.g. {"ignore", "group:tree"} or "default:dirt"
+int ModApiEnvMod::l_find_nodes_in_area_under_air(lua_State *L)
+{
+	/* Note: A similar but generalized (and therefore slower) version of this
+	 * function could be created -- e.g. find_nodes_in_area_under -- which
+	 * would accept a node name (or ID?) or list of names that the "above node"
+	 * should be.
+	 * TODO
+	 */
+
+	GET_ENV_PTR;
+
+	INodeDefManager *ndef = getServer(L)->ndef();
+	v3s16 minp = read_v3s16(L, 1);
+	v3s16 maxp = read_v3s16(L, 2);
+	std::set<content_t> filter;
+
+	if (lua_istable(L, 3)) {
+		int table = 3;
+		lua_pushnil(L);
+		while(lua_next(L, table) != 0) {
+			// key at index -2 and value at index -1
+			luaL_checktype(L, -1, LUA_TSTRING);
+			ndef->getIds(lua_tostring(L, -1), filter);
+			// removes value, keeps key for next iteration
+			lua_pop(L, 1);
+		}
+	} else if (lua_isstring(L, 3)) {
 		ndef->getIds(lua_tostring(L, 3), filter);
 	}
 
 	lua_newtable(L);
 	u64 i = 0;
-	for(s16 x = minp.X; x <= maxp.X; x++)
-	for(s16 y = minp.Y; y <= maxp.Y; y++)
-	for(s16 z = minp.Z; z <= maxp.Z; z++) {
+	for (s16 x = minp.X; x <= maxp.X; x++)
+	for (s16 z = minp.Z; z <= maxp.Z; z++) {
+		s16 y = minp.Y;
 		v3s16 p(x, y, z);
 		content_t c = env->getMap().getNodeNoEx(p).getContent();
-		if(filter.count(c) != 0) {
-			push_v3s16(L, p);
-			lua_rawseti(L, -2, ++i);
+		for (; y <= maxp.Y; y++) {
+			v3s16 psurf(x, y + 1, z);
+			content_t csurf = env->getMap().getNodeNoEx(psurf).getContent();
+			if(c != CONTENT_AIR && csurf == CONTENT_AIR &&
+					filter.count(c) != 0) {
+				push_v3s16(L, v3s16(x, y, z));
+				lua_rawseti(L, -2, ++i);
+			}
+			c = csurf;
 		}
 	}
 	return 1;
@@ -702,10 +773,12 @@ int ModApiEnvMod::l_delete_area(lua_State *L)
 	for (s16 y = bpmin.Y; y <= bpmax.Y; y++)
 	for (s16 x = bpmin.X; x <= bpmax.X; x++) {
 		v3s16 bp(x, y, z);
-		if (map.deleteBlock(bp))
+		if (map.deleteBlock(bp)) {
+			env->setStaticForActiveObjectsInBlock(bp, false);
 			event.modified_blocks.insert(bp);
-		else
+		} else {
 			success = false;
+		}
 	}
 
 	map.dispatchEvent(&event);
@@ -872,6 +945,7 @@ void ModApiEnvMod::Initialize(lua_State *L, int top)
 	API_FCT(set_node_level);
 	API_FCT(add_node_level);
 	API_FCT(add_entity);
+	API_FCT(find_nodes_with_meta);
 	API_FCT(get_meta);
 	API_FCT(get_node_timer);
 	API_FCT(get_player_by_name);
@@ -881,6 +955,7 @@ void ModApiEnvMod::Initialize(lua_State *L, int top)
 	API_FCT(get_gametime);
 	API_FCT(find_node_near);
 	API_FCT(find_nodes_in_area);
+	API_FCT(find_nodes_in_area_under_air);
 	API_FCT(delete_area);
 	API_FCT(get_perlin);
 	API_FCT(get_perlin_map);
