@@ -241,6 +241,53 @@ int InvRef::l_set_list(lua_State *L)
 	return 0;
 }
 
+// get_lists(self) -> list of InventoryLists
+int InvRef::l_get_lists(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	InvRef *ref = checkobject(L, 1);
+	Inventory *inv = getinv(L, ref);
+	if (!inv) {
+		return 0;
+	}
+	std::vector<const InventoryList*> lists = inv->getLists();
+	std::vector<const InventoryList*>::iterator iter = lists.begin();
+	lua_createtable(L, 0, lists.size());
+	for (; iter != lists.end(); iter++) {
+		const char* name = (*iter)->getName().c_str();
+		lua_pushstring(L, name);
+		push_inventory_list(L, inv, name);
+		lua_rawset(L, -3);
+	}
+	return 1;
+}
+
+// set_lists(self, lists)
+int InvRef::l_set_lists(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	InvRef *ref = checkobject(L, 1);
+	Inventory *inv = getinv(L, ref);
+	if (!inv) {
+		return 0;
+	}
+
+	// Make a temporary inventory in case reading fails
+	Inventory *tempInv(inv);
+	tempInv->clear();
+
+	Server *server = getServer(L);
+
+	lua_pushnil(L);
+	while (lua_next(L, 2)) {
+		const char *listname = lua_tostring(L, -2);
+		read_inventory_list(L, -1, tempInv, listname, server);
+		lua_pop(L, 1);
+	}
+	inv = tempInv;
+	return 0;
+}
+
 // add_item(self, listname, itemstack or itemstring or table or nil) -> itemstack
 // Returns the leftover stack
 int InvRef::l_add_item(lua_State *L)
@@ -315,7 +362,7 @@ int InvRef::l_remove_item(lua_State *L)
 	return 1;
 }
 
-// get_location() -> location (like minetest.get_inventory(location))
+// get_location() -> location (like get_inventory(location))
 int InvRef::l_get_location(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
@@ -426,6 +473,8 @@ const luaL_reg InvRef::methods[] = {
 	luamethod(InvRef, set_stack),
 	luamethod(InvRef, get_list),
 	luamethod(InvRef, set_list),
+	luamethod(InvRef, get_lists),
+	luamethod(InvRef, set_lists),
 	luamethod(InvRef, add_item),
 	luamethod(InvRef, room_for_item),
 	luamethod(InvRef, contains_item),
