@@ -23,36 +23,10 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "common/c_content.h"
 #include "log.h"
 
-// garbage collector
-int LuaPerlinNoise::gc_object(lua_State *L)
-{
-	LuaPerlinNoise *o = *(LuaPerlinNoise **)(lua_touserdata(L, 1));
-	delete o;
-	return 0;
-}
-
-
-int LuaPerlinNoise::l_get2d(lua_State *L)
-{
-	NO_MAP_LOCK_REQUIRED;
-	LuaPerlinNoise *o = checkobject(L, 1);
-	v2f p = read_v2f(L, 2);
-	lua_Number val = NoisePerlin2D(&o->np, p.X, p.Y, 0);
-	lua_pushnumber(L, val);
-	return 1;
-}
-
-
-int LuaPerlinNoise::l_get3d(lua_State *L)
-{
-	NO_MAP_LOCK_REQUIRED;
-	LuaPerlinNoise *o = checkobject(L, 1);
-	v3f p = read_v3f(L, 2);
-	lua_Number val = NoisePerlin3D(&o->np, p.X, p.Y, p.Z, 0);
-	lua_pushnumber(L, val);
-	return 1;
-}
-
+///////////////////////////////////////
+/*
+  LuaPerlinNoise
+*/
 
 LuaPerlinNoise::LuaPerlinNoise(NoiseParams *params) :
 	np(*params)
@@ -65,8 +39,28 @@ LuaPerlinNoise::~LuaPerlinNoise()
 }
 
 
-// LuaPerlinNoise(seed, octaves, persistence, scale)
-// Creates an LuaPerlinNoise and leaves it on top of stack
+int LuaPerlinNoise::l_get2d(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	LuaPerlinNoise *o = checkobject(L, 1);
+	v2f p = check_v2f(L, 2);
+	lua_Number val = NoisePerlin2D(&o->np, p.X, p.Y, 0);
+	lua_pushnumber(L, val);
+	return 1;
+}
+
+
+int LuaPerlinNoise::l_get3d(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	LuaPerlinNoise *o = checkobject(L, 1);
+	v3f p = check_v3f(L, 2);
+	lua_Number val = NoisePerlin3D(&o->np, p.X, p.Y, p.Z, 0);
+	lua_pushnumber(L, val);
+	return 1;
+}
+
+
 int LuaPerlinNoise::create_object(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
@@ -91,14 +85,22 @@ int LuaPerlinNoise::create_object(lua_State *L)
 }
 
 
-LuaPerlinNoise* LuaPerlinNoise::checkobject(lua_State *L, int narg)
+int LuaPerlinNoise::gc_object(lua_State *L)
+{
+	LuaPerlinNoise *o = *(LuaPerlinNoise **)(lua_touserdata(L, 1));
+	delete o;
+	return 0;
+}
+
+
+LuaPerlinNoise *LuaPerlinNoise::checkobject(lua_State *L, int narg)
 {
 	NO_MAP_LOCK_REQUIRED;
 	luaL_checktype(L, narg, LUA_TUSERDATA);
 	void *ud = luaL_checkudata(L, narg, className);
 	if (!ud)
 		luaL_typerror(L, narg, className);
-	return *(LuaPerlinNoise**)ud;  // unbox pointer
+	return *(LuaPerlinNoise **)ud;
 }
 
 
@@ -111,7 +113,7 @@ void LuaPerlinNoise::Register(lua_State *L)
 
 	lua_pushliteral(L, "__metatable");
 	lua_pushvalue(L, methodtable);
-	lua_settable(L, metatable);  // hide metatable from Lua getmetatable()
+	lua_settable(L, metatable);
 
 	lua_pushliteral(L, "__index");
 	lua_pushvalue(L, methodtable);
@@ -121,12 +123,11 @@ void LuaPerlinNoise::Register(lua_State *L)
 	lua_pushcfunction(L, gc_object);
 	lua_settable(L, metatable);
 
-	lua_pop(L, 1);  // drop metatable
+	lua_pop(L, 1);
 
-	luaL_openlib(L, 0, methods, 0);  // fill methodtable
-	lua_pop(L, 1);  // drop methodtable
+	luaL_openlib(L, 0, methods, 0);
+	lua_pop(L, 1);
 
-	// Can be created from Lua (PerlinNoise(seed, octaves, persistence)
 	lua_register(L, className, create_object);
 }
 
@@ -138,118 +139,10 @@ const luaL_reg LuaPerlinNoise::methods[] = {
 	{0,0}
 };
 
-
+///////////////////////////////////////
 /*
-  PerlinNoiseMap
- */
-
-int LuaPerlinNoiseMap::gc_object(lua_State *L)
-{
-	LuaPerlinNoiseMap *o = *(LuaPerlinNoiseMap **)(lua_touserdata(L, 1));
-	delete o;
-	return 0;
-}
-
-
-int LuaPerlinNoiseMap::l_get2dMap(lua_State *L)
-{
-	NO_MAP_LOCK_REQUIRED;
-	size_t i = 0;
-
-	LuaPerlinNoiseMap *o = checkobject(L, 1);
-	v2f p = read_v2f(L, 2);
-
-	Noise *n = o->noise;
-	n->perlinMap2D(p.X, p.Y);
-
-	lua_newtable(L);
-	for (int y = 0; y != n->sy; y++) {
-		lua_newtable(L);
-		for (int x = 0; x != n->sx; x++) {
-			lua_pushnumber(L, n->result[i++]);
-			lua_rawseti(L, -2, x + 1);
-		}
-		lua_rawseti(L, -2, y + 1);
-	}
-	return 1;
-}
-
-
-int LuaPerlinNoiseMap::l_get2dMap_flat(lua_State *L)
-{
-	NO_MAP_LOCK_REQUIRED;
-
-	LuaPerlinNoiseMap *o = checkobject(L, 1);
-	v2f p = read_v2f(L, 2);
-
-	Noise *n = o->noise;
-	n->perlinMap2D(p.X, p.Y);
-
-	size_t maplen = n->sx * n->sy;
-
-	lua_newtable(L);
-	for (size_t i = 0; i != maplen; i++) {
-		lua_pushnumber(L, n->result[i]);
-		lua_rawseti(L, -2, i + 1);
-	}
-	return 1;
-}
-
-
-int LuaPerlinNoiseMap::l_get3dMap(lua_State *L)
-{
-	NO_MAP_LOCK_REQUIRED;
-	size_t i = 0;
-
-	LuaPerlinNoiseMap *o = checkobject(L, 1);
-	v3f p = read_v3f(L, 2);
-
-	if (!o->m_is3d)
-		return 0;
-
-	Noise *n = o->noise;
-	n->perlinMap3D(p.X, p.Y, p.Z);
-
-	lua_newtable(L);
-	for (int z = 0; z != n->sz; z++) {
-		lua_newtable(L);
-		for (int y = 0; y != n->sy; y++) {
-			lua_newtable(L);
-			for (int x = 0; x != n->sx; x++) {
-				lua_pushnumber(L, n->result[i++]);
-				lua_rawseti(L, -2, x + 1);
-			}
-			lua_rawseti(L, -2, y + 1);
-		}
-		lua_rawseti(L, -2, z + 1);
-	}
-	return 1;
-}
-
-
-int LuaPerlinNoiseMap::l_get3dMap_flat(lua_State *L)
-{
-	NO_MAP_LOCK_REQUIRED;
-
-	LuaPerlinNoiseMap *o = checkobject(L, 1);
-	v3f p = read_v3f(L, 2);
-
-	if (!o->m_is3d)
-		return 0;
-
-	Noise *n = o->noise;
-	n->perlinMap3D(p.X, p.Y, p.Z);
-
-	size_t maplen = n->sx * n->sy * n->sz;
-
-	lua_newtable(L);
-	for (size_t i = 0; i != maplen; i++) {
-		lua_pushnumber(L, n->result[i]);
-		lua_rawseti(L, -2, i + 1);
-	}
-	return 1;
-}
-
+  LuaPerlinNoiseMap
+*/
 
 LuaPerlinNoiseMap::LuaPerlinNoiseMap(NoiseParams *params, int seed, v3s16 size)
 {
@@ -269,8 +162,171 @@ LuaPerlinNoiseMap::~LuaPerlinNoiseMap()
 }
 
 
-// LuaPerlinNoiseMap(np, size)
-// Creates an LuaPerlinNoiseMap and leaves it on top of stack
+int LuaPerlinNoiseMap::l_get2dMap(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	size_t i = 0;
+
+	LuaPerlinNoiseMap *o = checkobject(L, 1);
+	v2f p = check_v2f(L, 2);
+
+	Noise *n = o->noise;
+	n->perlinMap2D(p.X, p.Y);
+
+	lua_newtable(L);
+	for (u32 y = 0; y != n->sy; y++) {
+		lua_newtable(L);
+		for (u32 x = 0; x != n->sx; x++) {
+			lua_pushnumber(L, n->result[i++]);
+			lua_rawseti(L, -2, x + 1);
+		}
+		lua_rawseti(L, -2, y + 1);
+	}
+	return 1;
+}
+
+
+int LuaPerlinNoiseMap::l_get2dMap_flat(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	LuaPerlinNoiseMap *o = checkobject(L, 1);
+	v2f p                = check_v2f(L, 2);
+	bool use_buffer      = lua_istable(L, 3);
+
+	Noise *n = o->noise;
+	n->perlinMap2D(p.X, p.Y);
+
+	size_t maplen = n->sx * n->sy;
+
+	if (use_buffer)
+		lua_pushvalue(L, 3);
+	else
+		lua_newtable(L);
+
+	for (size_t i = 0; i != maplen; i++) {
+		lua_pushnumber(L, n->result[i]);
+		lua_rawseti(L, -2, i + 1);
+	}
+	return 1;
+}
+
+
+int LuaPerlinNoiseMap::l_get3dMap(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	size_t i = 0;
+
+	LuaPerlinNoiseMap *o = checkobject(L, 1);
+	v3f p = check_v3f(L, 2);
+
+	if (!o->m_is3d)
+		return 0;
+
+	Noise *n = o->noise;
+	n->perlinMap3D(p.X, p.Y, p.Z);
+
+	lua_newtable(L);
+	for (u32 z = 0; z != n->sz; z++) {
+		lua_newtable(L);
+		for (u32 y = 0; y != n->sy; y++) {
+			lua_newtable(L);
+			for (u32 x = 0; x != n->sx; x++) {
+				lua_pushnumber(L, n->result[i++]);
+				lua_rawseti(L, -2, x + 1);
+			}
+			lua_rawseti(L, -2, y + 1);
+		}
+		lua_rawseti(L, -2, z + 1);
+	}
+	return 1;
+}
+
+
+int LuaPerlinNoiseMap::l_get3dMap_flat(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	LuaPerlinNoiseMap *o = checkobject(L, 1);
+	v3f p                = check_v3f(L, 2);
+	bool use_buffer      = lua_istable(L, 3);
+
+	if (!o->m_is3d)
+		return 0;
+
+	Noise *n = o->noise;
+	n->perlinMap3D(p.X, p.Y, p.Z);
+
+	size_t maplen = n->sx * n->sy * n->sz;
+
+	if (use_buffer)
+		lua_pushvalue(L, 3);
+	else
+		lua_newtable(L);
+
+	for (size_t i = 0; i != maplen; i++) {
+		lua_pushnumber(L, n->result[i]);
+		lua_rawseti(L, -2, i + 1);
+	}
+	return 1;
+}
+
+
+int LuaPerlinNoiseMap::l_calc2dMap(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	LuaPerlinNoiseMap *o = checkobject(L, 1);
+	v2f p                = check_v2f(L, 2);
+
+	Noise *n = o->noise;
+	n->perlinMap2D(p.X, p.Y);
+
+	return 0;
+}
+
+int LuaPerlinNoiseMap::l_calc3dMap(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	LuaPerlinNoiseMap *o = checkobject(L, 1);
+	v3f p                = check_v3f(L, 2);
+
+	if (!o->m_is3d)
+		return 0;
+
+	Noise *n = o->noise;
+	n->perlinMap3D(p.X, p.Y, p.Z);
+
+	return 0;
+}
+
+
+int LuaPerlinNoiseMap::l_getMapSlice(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	LuaPerlinNoiseMap *o = checkobject(L, 1);
+	v3s16 slice_offset   = read_v3s16(L, 2);
+	v3s16 slice_size     = read_v3s16(L, 3);
+	bool use_buffer      = lua_istable(L, 4);
+
+	Noise *n = o->noise;
+
+	if (use_buffer)
+		lua_pushvalue(L, 3);
+	else
+		lua_newtable(L);
+
+	write_array_slice_float(L, lua_gettop(L), n->result,
+		v3u16(n->sx, n->sy, n->sz),
+		v3u16(slice_offset.X, slice_offset.Y, slice_offset.Z),
+		v3u16(slice_size.X, slice_size.Y, slice_size.Z));
+
+	return 1;
+}
+
+
 int LuaPerlinNoiseMap::create_object(lua_State *L)
 {
 	NoiseParams np;
@@ -286,6 +342,14 @@ int LuaPerlinNoiseMap::create_object(lua_State *L)
 }
 
 
+int LuaPerlinNoiseMap::gc_object(lua_State *L)
+{
+	LuaPerlinNoiseMap *o = *(LuaPerlinNoiseMap **)(lua_touserdata(L, 1));
+	delete o;
+	return 0;
+}
+
+
 LuaPerlinNoiseMap *LuaPerlinNoiseMap::checkobject(lua_State *L, int narg)
 {
 	luaL_checktype(L, narg, LUA_TUSERDATA);
@@ -294,7 +358,7 @@ LuaPerlinNoiseMap *LuaPerlinNoiseMap::checkobject(lua_State *L, int narg)
 	if (!ud)
 		luaL_typerror(L, narg, className);
 
-	return *(LuaPerlinNoiseMap **)ud;  // unbox pointer
+	return *(LuaPerlinNoiseMap **)ud;
 }
 
 
@@ -307,7 +371,7 @@ void LuaPerlinNoiseMap::Register(lua_State *L)
 
 	lua_pushliteral(L, "__metatable");
 	lua_pushvalue(L, methodtable);
-	lua_settable(L, metatable);  // hide metatable from Lua getmetatable()
+	lua_settable(L, metatable);
 
 	lua_pushliteral(L, "__index");
 	lua_pushvalue(L, methodtable);
@@ -317,12 +381,11 @@ void LuaPerlinNoiseMap::Register(lua_State *L)
 	lua_pushcfunction(L, gc_object);
 	lua_settable(L, metatable);
 
-	lua_pop(L, 1);  // drop metatable
+	lua_pop(L, 1);
 
-	luaL_openlib(L, 0, methods, 0);  // fill methodtable
-	lua_pop(L, 1);  // drop methodtable
+	luaL_openlib(L, 0, methods, 0);
+	lua_pop(L, 1);
 
-	// Can be created from Lua (PerlinNoiseMap(np, size)
 	lua_register(L, className, create_object);
 }
 
@@ -331,37 +394,31 @@ const char LuaPerlinNoiseMap::className[] = "PerlinNoiseMap";
 const luaL_reg LuaPerlinNoiseMap::methods[] = {
 	luamethod(LuaPerlinNoiseMap, get2dMap),
 	luamethod(LuaPerlinNoiseMap, get2dMap_flat),
+	luamethod(LuaPerlinNoiseMap, calc2dMap),
 	luamethod(LuaPerlinNoiseMap, get3dMap),
 	luamethod(LuaPerlinNoiseMap, get3dMap_flat),
+	luamethod(LuaPerlinNoiseMap, calc3dMap),
+	luamethod(LuaPerlinNoiseMap, getMapSlice),
 	{0,0}
 };
 
+///////////////////////////////////////
 /*
 	LuaPseudoRandom
 */
 
-// garbage collector
-int LuaPseudoRandom::gc_object(lua_State *L)
-{
-	LuaPseudoRandom *o = *(LuaPseudoRandom **)(lua_touserdata(L, 1));
-	delete o;
-	return 0;
-}
-
-
-// next(self, min=0, max=32767) -> get next value
 int LuaPseudoRandom::l_next(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
 	LuaPseudoRandom *o = checkobject(L, 1);
 	int min = 0;
 	int max = 32767;
-	lua_settop(L, 3); // Fill 2 and 3 with nil if they don't exist
-	if(!lua_isnil(L, 2))
+	lua_settop(L, 3);
+	if (lua_isnumber(L, 2))
 		min = luaL_checkinteger(L, 2);
-	if(!lua_isnil(L, 3))
+	if (lua_isnumber(L, 3))
 		max = luaL_checkinteger(L, 3);
-	if(max < min){
+	if (max < min) {
 		errorstream<<"PseudoRandom.next(): max="<<max<<" min="<<min<<std::endl;
 		throw LuaError("PseudoRandom.next(): max < min");
 	}
@@ -378,30 +435,6 @@ int LuaPseudoRandom::l_next(lua_State *L)
 }
 
 
-LuaPseudoRandom::LuaPseudoRandom(int seed):
-	m_pseudo(seed)
-{
-}
-
-
-LuaPseudoRandom::~LuaPseudoRandom()
-{
-}
-
-
-const PseudoRandom& LuaPseudoRandom::getItem() const
-{
-	return m_pseudo;
-}
-
-PseudoRandom& LuaPseudoRandom::getItem()
-{
-	return m_pseudo;
-}
-
-
-// LuaPseudoRandom(seed)
-// Creates an LuaPseudoRandom and leaves it on top of stack
 int LuaPseudoRandom::create_object(lua_State *L)
 {
 	int seed = luaL_checknumber(L, 1);
@@ -413,13 +446,21 @@ int LuaPseudoRandom::create_object(lua_State *L)
 }
 
 
-LuaPseudoRandom* LuaPseudoRandom::checkobject(lua_State *L, int narg)
+int LuaPseudoRandom::gc_object(lua_State *L)
+{
+	LuaPseudoRandom *o = *(LuaPseudoRandom **)(lua_touserdata(L, 1));
+	delete o;
+	return 0;
+}
+
+
+LuaPseudoRandom *LuaPseudoRandom::checkobject(lua_State *L, int narg)
 {
 	luaL_checktype(L, narg, LUA_TUSERDATA);
 	void *ud = luaL_checkudata(L, narg, className);
 	if (!ud)
 		luaL_typerror(L, narg, className);
-	return *(LuaPseudoRandom**)ud;  // unbox pointer
+	return *(LuaPseudoRandom **)ud;
 }
 
 
@@ -432,7 +473,7 @@ void LuaPseudoRandom::Register(lua_State *L)
 
 	lua_pushliteral(L, "__metatable");
 	lua_pushvalue(L, methodtable);
-	lua_settable(L, metatable);  // hide metatable from Lua getmetatable()
+	lua_settable(L, metatable);
 
 	lua_pushliteral(L, "__index");
 	lua_pushvalue(L, methodtable);
@@ -442,12 +483,11 @@ void LuaPseudoRandom::Register(lua_State *L)
 	lua_pushcfunction(L, gc_object);
 	lua_settable(L, metatable);
 
-	lua_pop(L, 1);  // drop metatable
+	lua_pop(L, 1);
 
-	luaL_openlib(L, 0, methods, 0);  // fill methodtable
-	lua_pop(L, 1);  // drop methodtable
+	luaL_openlib(L, 0, methods, 0);
+	lua_pop(L, 1);
 
-	// Can be created from Lua (LuaPseudoRandom(seed))
 	lua_register(L, className, create_object);
 }
 
@@ -457,3 +497,101 @@ const luaL_reg LuaPseudoRandom::methods[] = {
 	luamethod(LuaPseudoRandom, next),
 	{0,0}
 };
+
+///////////////////////////////////////
+/*
+	LuaPcgRandom
+*/
+
+int LuaPcgRandom::l_next(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	LuaPcgRandom *o = checkobject(L, 1);
+	u32 min = lua_isnumber(L, 2) ? lua_tointeger(L, 2) : o->m_rnd.RANDOM_MIN;
+	u32 max = lua_isnumber(L, 3) ? lua_tointeger(L, 3) : o->m_rnd.RANDOM_MAX;
+
+	lua_pushinteger(L, o->m_rnd.range(min, max));
+	return 1;
+}
+
+
+int LuaPcgRandom::l_rand_normal_dist(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	LuaPcgRandom *o = checkobject(L, 1);
+	u32 min = lua_isnumber(L, 2) ? lua_tointeger(L, 2) : o->m_rnd.RANDOM_MIN;
+	u32 max = lua_isnumber(L, 3) ? lua_tointeger(L, 3) : o->m_rnd.RANDOM_MAX;
+	int num_trials = lua_isnumber(L, 4) ? lua_tointeger(L, 4) : 6;
+
+	lua_pushinteger(L, o->m_rnd.randNormalDist(min, max, num_trials));
+	return 1;
+}
+
+
+int LuaPcgRandom::create_object(lua_State *L)
+{
+	lua_Integer seed = luaL_checknumber(L, 1);
+	LuaPcgRandom *o  = lua_isnumber(L, 2) ?
+		new LuaPcgRandom(seed, lua_tointeger(L, 2)) :
+		new LuaPcgRandom(seed);
+	*(void **)(lua_newuserdata(L, sizeof(void *))) = o;
+	luaL_getmetatable(L, className);
+	lua_setmetatable(L, -2);
+	return 1;
+}
+
+
+int LuaPcgRandom::gc_object(lua_State *L)
+{
+	LuaPcgRandom *o = *(LuaPcgRandom **)(lua_touserdata(L, 1));
+	delete o;
+	return 0;
+}
+
+
+LuaPcgRandom *LuaPcgRandom::checkobject(lua_State *L, int narg)
+{
+	luaL_checktype(L, narg, LUA_TUSERDATA);
+	void *ud = luaL_checkudata(L, narg, className);
+	if (!ud)
+		luaL_typerror(L, narg, className);
+	return *(LuaPcgRandom **)ud;
+}
+
+
+void LuaPcgRandom::Register(lua_State *L)
+{
+	lua_newtable(L);
+	int methodtable = lua_gettop(L);
+	luaL_newmetatable(L, className);
+	int metatable = lua_gettop(L);
+
+	lua_pushliteral(L, "__metatable");
+	lua_pushvalue(L, methodtable);
+	lua_settable(L, metatable);
+
+	lua_pushliteral(L, "__index");
+	lua_pushvalue(L, methodtable);
+	lua_settable(L, metatable);
+
+	lua_pushliteral(L, "__gc");
+	lua_pushcfunction(L, gc_object);
+	lua_settable(L, metatable);
+
+	lua_pop(L, 1);
+
+	luaL_openlib(L, 0, methods, 0);
+	lua_pop(L, 1);
+
+	lua_register(L, className, create_object);
+}
+
+
+const char LuaPcgRandom::className[] = "PcgRandom";
+const luaL_reg LuaPcgRandom::methods[] = {
+	luamethod(LuaPcgRandom, next),
+	luamethod(LuaPcgRandom, rand_normal_dist),
+	{0,0}
+};
