@@ -78,7 +78,7 @@ private:
 	// is_yes(arg)
 	static int l_is_yes(lua_State *L);
 
-	// get_scriptdir()
+	// get_builtin_path()
 	static int l_get_builtin_path(lua_State *L);
 
 	// compress(data, method, ...)
@@ -87,6 +87,15 @@ private:
 	// decompress(data, method, ...)
 	static int l_decompress(lua_State *L);
 
+	// mkdir(path)
+	static int l_mkdir(lua_State *L);
+
+	// get_dir_list(path, is_dir)
+	static int l_get_dir_list(lua_State *L);
+
+	// request_insecure_environment()
+	static int l_request_insecure_environment(lua_State *L);
+
 public:
 	static void Initialize(lua_State *L, int top);
 
@@ -95,3 +104,4 @@ public:
 };
 
 #endif /* L_UTIL_H_ */
+
