@@ -44,10 +44,10 @@ int LuaVoxelManip::gc_object(lua_State *L)
 int LuaVoxelManip::l_read_from_map(lua_State *L)
 {
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
-	v3s16 bp1 = getNodeBlockPos(read_v3s16(L, 2));
-	v3s16 bp2 = getNodeBlockPos(read_v3s16(L, 3));
+	v3s16 bp1 = getNodeBlockPos(check_v3s16(L, 2));
+	v3s16 bp2 = getNodeBlockPos(check_v3s16(L, 3));
 	sortBoxVerticies(bp1, bp2);
 
 	vm->initialEmerge(bp1, bp2);
@@ -63,12 +63,18 @@ int LuaVoxelManip::l_get_data(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	bool use_buffer  = lua_istable(L, 2);
 
-	int volume = vm->m_area.getVolume();
+	MMVManip *vm = o->vm;
 
-	lua_newtable(L);
-	for (int i = 0; i != volume; i++) {
+	u32 volume = vm->m_area.getVolume();
+
+	if (use_buffer)
+		lua_pushvalue(L, 2);
+	else
+		lua_newtable(L);
+
+	for (u32 i = 0; i != volume; i++) {
 		lua_Integer cid = vm->m_data[i].getContent();
 		lua_pushinteger(L, cid);
 		lua_rawseti(L, -2, i + 1);
@@ -82,13 +88,13 @@ int LuaVoxelManip::l_set_data(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
 	if (!lua_istable(L, 2))
 		return 0;
 
-	int volume = vm->m_area.getVolume();
-	for (int i = 0; i != volume; i++) {
+	u32 volume = vm->m_area.getVolume();
+	for (u32 i = 0; i != volume; i++) {
 		lua_rawgeti(L, 2, i + 1);
 		content_t c = lua_tointeger(L, -1);
 
@@ -103,7 +109,7 @@ int LuaVoxelManip::l_set_data(lua_State *L)
 int LuaVoxelManip::l_write_to_map(lua_State *L)
 {
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
 	vm->blitBackAll(&o->modified_blocks);
 
@@ -116,7 +122,7 @@ int LuaVoxelManip::l_get_node_at(lua_State *L)
 	GET_ENV_PTR;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	v3s16 pos        = read_v3s16(L, 2);
+	v3s16 pos        = check_v3s16(L, 2);
 
 	pushnode(L, o->vm->getNodeNoExNoEmerge(pos), env->getGameDef()->ndef());
 	return 1;
@@ -128,7 +134,7 @@ int LuaVoxelManip::l_set_node_at(lua_State *L)
 	GET_ENV_PTR;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	v3s16 pos        = read_v3s16(L, 2);
+	v3s16 pos        = check_v3s16(L, 2);
 	MapNode n        = readnode(L, 3, env->getGameDef()->ndef());
 
 	o->vm->setNodeNoEmerge(pos, n);
@@ -144,7 +150,7 @@ int LuaVoxelManip::l_update_liquids(lua_State *L)
 
 	Map *map = &(env->getMap());
 	INodeDefManager *ndef = getServer(L)->getNodeDefManager();
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
 	Mapgen mg;
 	mg.vm   = vm;
@@ -166,20 +172,24 @@ int LuaVoxelManip::l_calc_lighting(lua_State *L)
 
 	INodeDefManager *ndef = getServer(L)->getNodeDefManager();
 	EmergeManager *emerge = getServer(L)->getEmergeManager();
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
-	v3s16 p1 = lua_istable(L, 2) ? read_v3s16(L, 2) :
-		vm->m_area.MinEdge + v3s16(0, 1, 0) * MAP_BLOCKSIZE;
-	v3s16 p2 = lua_istable(L, 3) ? read_v3s16(L, 3) :
-		vm->m_area.MaxEdge - v3s16(0, 1, 0) * MAP_BLOCKSIZE;
-	sortBoxVerticies(p1, p2);
+	v3s16 yblock = v3s16(0, 1, 0) * MAP_BLOCKSIZE;
+	v3s16 fpmin  = vm->m_area.MinEdge;
+	v3s16 fpmax  = vm->m_area.MaxEdge;
+	v3s16 pmin   = lua_istable(L, 2) ? check_v3s16(L, 2) : fpmin + yblock;
+	v3s16 pmax   = lua_istable(L, 3) ? check_v3s16(L, 3) : fpmax - yblock;
+
+	sortBoxVerticies(pmin, pmax);
+	if (!vm->m_area.contains(VoxelArea(pmin, pmax)))
+		throw LuaError("Specified voxel area out of VoxelManipulator bounds");
 
 	Mapgen mg;
 	mg.vm          = vm;
 	mg.ndef        = ndef;
 	mg.water_level = emerge->params.water_level;
 
-	mg.calcLighting(p1, p2);
+	mg.calcLighting(pmin, pmax, fpmin, fpmax);
 
 	return 0;
 }
@@ -199,18 +209,20 @@ int LuaVoxelManip::l_set_lighting(lua_State *L)
 	light  = (getintfield_default(L, 2, "day",   0) & 0x0F);
 	light |= (getintfield_default(L, 2, "night", 0) & 0x0F) << 4;
 
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
-	v3s16 p1 = lua_istable(L, 3) ? read_v3s16(L, 3) :
-		vm->m_area.MinEdge + v3s16(0, 1, 0) * MAP_BLOCKSIZE;
-	v3s16 p2 = lua_istable(L, 4) ? read_v3s16(L, 4) :
-		vm->m_area.MaxEdge - v3s16(0, 1, 0) * MAP_BLOCKSIZE;
-	sortBoxVerticies(p1, p2);
+	v3s16 yblock = v3s16(0, 1, 0) * MAP_BLOCKSIZE;
+	v3s16 pmin = lua_istable(L, 3) ? check_v3s16(L, 3) : vm->m_area.MinEdge + yblock;
+	v3s16 pmax = lua_istable(L, 4) ? check_v3s16(L, 4) : vm->m_area.MaxEdge - yblock;
+
+	sortBoxVerticies(pmin, pmax);
+	if (!vm->m_area.contains(VoxelArea(pmin, pmax)))
+		throw LuaError("Specified voxel area out of VoxelManipulator bounds");
 
 	Mapgen mg;
 	mg.vm = vm;
 
-	mg.setLighting(p1, p2, light);
+	mg.setLighting(light, pmin, pmax);
 
 	return 0;
 }
@@ -220,12 +232,12 @@ int LuaVoxelManip::l_get_light_data(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
-	int volume = vm->m_area.getVolume();
+	u32 volume = vm->m_area.getVolume();
 
 	lua_newtable(L);
-	for (int i = 0; i != volume; i++) {
+	for (u32 i = 0; i != volume; i++) {
 		lua_Integer light = vm->m_data[i].param1;
 		lua_pushinteger(L, light);
 		lua_rawseti(L, -2, i + 1);
@@ -239,13 +251,13 @@ int LuaVoxelManip::l_set_light_data(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
 	if (!lua_istable(L, 2))
 		return 0;
 
-	int volume = vm->m_area.getVolume();
-	for (int i = 0; i != volume; i++) {
+	u32 volume = vm->m_area.getVolume();
+	for (u32 i = 0; i != volume; i++) {
 		lua_rawgeti(L, 2, i + 1);
 		u8 light = lua_tointeger(L, -1);
 
@@ -262,12 +274,12 @@ int LuaVoxelManip::l_get_param2_data(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
-	int volume = vm->m_area.getVolume();
+	u32 volume = vm->m_area.getVolume();
 
 	lua_newtable(L);
-	for (int i = 0; i != volume; i++) {
+	for (u32 i = 0; i != volume; i++) {
 		lua_Integer param2 = vm->m_data[i].param2;
 		lua_pushinteger(L, param2);
 		lua_rawseti(L, -2, i + 1);
@@ -281,13 +293,13 @@ int LuaVoxelManip::l_set_param2_data(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
 	if (!lua_istable(L, 2))
 		return 0;
 
-	int volume = vm->m_area.getVolume();
-	for (int i = 0; i != volume; i++) {
+	u32 volume = vm->m_area.getVolume();
+	for (u32 i = 0; i != volume; i++) {
 		lua_rawgeti(L, 2, i + 1);
 		u8 param2 = lua_tointeger(L, -1);
 
@@ -338,14 +350,24 @@ int LuaVoxelManip::l_was_modified(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 
 	LuaVoxelManip *o = checkobject(L, 1);
-	ManualMapVoxelManipulator *vm = o->vm;
+	MMVManip *vm = o->vm;
 
 	lua_pushboolean(L, vm->m_is_dirty);
 
 	return 1;
 }
 
-LuaVoxelManip::LuaVoxelManip(ManualMapVoxelManipulator *mmvm, bool is_mg_vm)
+int LuaVoxelManip::l_get_emerged_area(lua_State *L)
+{
+	LuaVoxelManip *o = checkobject(L, 1);
+
+	push_v3s16(L, o->vm->m_area.MinEdge);
+	push_v3s16(L, o->vm->m_area.MaxEdge);
+
+	return 2;
+}
+
+LuaVoxelManip::LuaVoxelManip(MMVManip *mmvm, bool is_mg_vm)
 {
 	this->vm           = mmvm;
 	this->is_mapgen_vm = is_mg_vm;
@@ -353,10 +375,21 @@ LuaVoxelManip::LuaVoxelManip(ManualMapVoxelManipulator *mmvm, bool is_mg_vm)
 
 LuaVoxelManip::LuaVoxelManip(Map *map)
 {
-	this->vm = new ManualMapVoxelManipulator(map);
+	this->vm = new MMVManip(map);
 	this->is_mapgen_vm = false;
 }
 
+LuaVoxelManip::LuaVoxelManip(Map *map, v3s16 p1, v3s16 p2)
+{
+	this->vm = new MMVManip(map);
+	this->is_mapgen_vm = false;
+
+	v3s16 bp1 = getNodeBlockPos(p1);
+	v3s16 bp2 = getNodeBlockPos(p2);
+	sortBoxVerticies(bp1, bp2);
+	vm->initialEmerge(bp1, bp2);
+}
+
 LuaVoxelManip::~LuaVoxelManip()
 {
 	if (!is_mapgen_vm)
@@ -374,7 +407,9 @@ int LuaVoxelManip::create_object(lua_State *L)
 		return 0;
 
 	Map *map = &(env->getMap());
-	LuaVoxelManip *o = new LuaVoxelManip(map);
+	LuaVoxelManip *o = (lua_istable(L, 1) && lua_istable(L, 2)) ?
+		new LuaVoxelManip(map, check_v3s16(L, 1), check_v3s16(L, 2)) :
+		new LuaVoxelManip(map);
 
 	*(void **)(lua_newuserdata(L, sizeof(void *))) = o;
 	luaL_getmetatable(L, className);
@@ -440,5 +475,6 @@ const luaL_reg LuaVoxelManip::methods[] = {
 	luamethod(LuaVoxelManip, get_param2_data),
 	luamethod(LuaVoxelManip, set_param2_data),
 	luamethod(LuaVoxelManip, was_modified),
+	luamethod(LuaVoxelManip, get_emerged_area),
 	{0,0}
 };
