@@ -26,14 +26,13 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 class Map;
 class MapBlock;
-class ManualMapVoxelManipulator;
+class MMVManip;
 
 /*
   VoxelManip
  */
 class LuaVoxelManip : public ModApiBase {
 private:
-	ManualMapVoxelManipulator *vm;
 	std::map<v3s16, MapBlock *> modified_blocks;
 	bool is_mapgen_vm;
 
@@ -47,6 +46,9 @@ private:
 	static int l_set_data(lua_State *L);
 	static int l_write_to_map(lua_State *L);
 
+	static int l_get_node_at(lua_State *L);
+	static int l_set_node_at(lua_State *L);
+
 	static int l_update_map(lua_State *L);
 	static int l_update_liquids(lua_State *L);
 
@@ -58,8 +60,14 @@ private:
 	static int l_get_param2_data(lua_State *L);
 	static int l_set_param2_data(lua_State *L);
 
+	static int l_was_modified(lua_State *L);
+	static int l_get_emerged_area(lua_State *L);
+
 public:
-	LuaVoxelManip(ManualMapVoxelManipulator *mmvm, bool is_mapgen_vm);
+	MMVManip *vm;
+
+	LuaVoxelManip(MMVManip *mmvm, bool is_mapgen_vm);
+	LuaVoxelManip(Map *map, v3s16 p1, v3s16 p2);
 	LuaVoxelManip(Map *map);
 	~LuaVoxelManip();
 
