@@ -38,7 +38,7 @@ namespace ServerList
 	std::vector<ServerListSpec> deSerializeJson(const std::string &liststring);
 	const std::string serializeJson(const std::vector<ServerListSpec> &serverlist);
 	#if USE_CURL
-	void sendAnnounce(const std::string &action,
+	void sendAnnounce(const std::string &action, const u16 port,
 			const std::vector<std::string> &clients_names = std::vector<std::string>(),
 			const double uptime = 0, const u32 game_time = 0,
 			const float lag = 0, const std::string &gameid = "",
