@@ -22,11 +22,15 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "filesys.h"
 #include "settings.h"
 #include "log.h"
-#ifndef SERVER
-#include "tile.h" // getImagePath
-#endif
+#include "strfnd.h"
+#include "defaultsettings.h"  // for override_default_settings
+#include "mapgen.h"  // for MapgenParams
 #include "util/string.h"
 
+#ifndef SERVER
+	#include "client/tile.h" // getImagePath
+#endif
+
 bool getGameMinetestConfig(const std::string &game_path, Settings &conf)
 {
 	std::string conf_path = game_path + DIR_DELIM + "minetest.conf";
@@ -59,6 +63,17 @@ struct GameFindPath
 	{}
 };
 
+Strfnd getSubgamePathEnv() {
+	std::string sp;
+	char *subgame_path = getenv("MINETEST_SUBGAME_PATH");
+
+	if(subgame_path) {
+		sp = std::string(subgame_path);
+	}
+
+	return Strfnd(sp);
+}
+
 SubgameSpec findSubgame(const std::string &id)
 {
 	if(id == "")
@@ -66,6 +81,17 @@ SubgameSpec findSubgame(const std::string &id)
 	std::string share = porting::path_share;
 	std::string user = porting::path_user;
 	std::vector<GameFindPath> find_paths;
+
+	Strfnd search_paths = getSubgamePathEnv();
+
+	while(!search_paths.atend()) {
+		std::string path = search_paths.next(":");
+		find_paths.push_back(GameFindPath(
+				path + DIR_DELIM + id, false));
+		find_paths.push_back(GameFindPath(
+				path + DIR_DELIM + id + "_game", false));
+	}
+
 	find_paths.push_back(GameFindPath(
 			user + DIR_DELIM + "games" + DIR_DELIM + id + "_game", true));
 	find_paths.push_back(GameFindPath(
@@ -129,6 +155,13 @@ std::set<std::string> getAvailableGameIds()
 	std::set<std::string> gamespaths;
 	gamespaths.insert(porting::path_share + DIR_DELIM + "games");
 	gamespaths.insert(porting::path_user + DIR_DELIM + "games");
+
+	Strfnd search_paths = getSubgamePathEnv();
+
+	while(!search_paths.atend()) {
+		gamespaths.insert(search_paths.next(":"));
+	}
+
 	for(std::set<std::string>::const_iterator i = gamespaths.begin();
 			i != gamespaths.end(); i++){
 		std::vector<fs::DirListNode> dirlist = fs::GetDirListing(*i);
@@ -233,24 +266,54 @@ std::vector<WorldSpec> getAvailableWorlds()
 	return worlds;
 }
 
-bool initializeWorld(const std::string &path, const std::string &gameid)
+bool loadGameConfAndInitWorld(const std::string &path, const SubgameSpec &gamespec)
 {
-	infostream<<"Initializing world at "<<path<<std::endl;
+	// Override defaults with those provided by the game.
+	// We clear and reload the defaults because the defaults
+	// might have been overridden by other subgame config
+	// files that were loaded before.
+	g_settings->clearDefaults();
+	set_default_settings(g_settings);
+	Settings game_defaults;
+	getGameMinetestConfig(gamespec.path, game_defaults);
+	override_default_settings(g_settings, &game_defaults);
+
+	infostream << "Initializing world at " << path << std::endl;
+
+	fs::CreateAllDirs(path);
+
 	// Create world.mt if does not already exist
-	std::string worldmt_path = path + DIR_DELIM + "world.mt";
-	if(!fs::PathExists(worldmt_path)){
-		infostream<<"Creating world.mt ("<<worldmt_path<<")"<<std::endl;
-		fs::CreateAllDirs(path);
+	std::string worldmt_path = path + DIR_DELIM "world.mt";
+	if (!fs::PathExists(worldmt_path)) {
 		std::ostringstream ss(std::ios_base::binary);
-		ss<<"gameid = "<<gameid<<
-#ifdef __ANDROID__
-				"\nbackend = leveldb\n";
-#else
-				"\nbackend = sqlite3\n";
-#endif
-		fs::safeWriteToFile(worldmt_path, ss.str());
+		ss << "gameid = " << gamespec.id
+			<< "\nbackend = sqlite3"
+			<< "\ncreative_mode = " << g_settings->get("creative_mode")
+			<< "\nenable_damage = " << g_settings->get("enable_damage")
+			<< "\n";
+		if (!fs::safeWriteToFile(worldmt_path, ss.str()))
+			return false;
+
+		infostream << "Wrote world.mt (" << worldmt_path << ")" << std::endl;
+	}
+
+	// Create map_meta.txt if does not already exist
+	std::string map_meta_path = path + DIR_DELIM + "map_meta.txt";
+	if (!fs::PathExists(map_meta_path)){
+		verbosestream << "Creating map_meta.txt (" << map_meta_path << ")" << std::endl;
+		fs::CreateAllDirs(path);
+		std::ostringstream oss(std::ios_base::binary);
+
+		Settings conf;
+		MapgenParams params;
+
+		params.load(*g_settings);
+		params.save(conf);
+		conf.writeLines(oss);
+		oss << "[end_of_params]\n";
+
+		fs::safeWriteToFile(map_meta_path, oss.str());
 	}
 	return true;
 }
 
-
